# Copyright (C) 2004 softLabs. All rights reserved
# Thomas-Morus Gasse 10/2/4, A--1130 Wien, Austria. smaili@softlabs.com
#
#++
# Name
#    CssTable.py
#
# Purpose
#    Generates CssTable's offsets
#
# Author
#    Idriz Smaili
#
# Revision Dates
#    19-Jul-2004 (IS) Creation
#    28-Oct-2004 (IS) 'check_version' added
#    28-Oct-2005 (IS) 'CSS_LDDATE' added to the 'css_types'
#--
#

import string
import sys
import re

from time import *

### file format version (ffv)
ffv_major = 0
ffv_minor = 1
ffv_patch = 1

class FClose (IOError) :
    pass
# end class FClose

class InvCssType (IOError) :
    pass
# end class FClose

class CssType :
    """ This class modulates the CssType (see gbase documentation)
    """
    ## define the allowed css types
    css_types = { "CSS_STRING" : "s",
                  "CSS_TIME"   : "lf",
                  "CSS_LONG"   : "ld",
                  "CSS_FLOAT"  : "f",
                  "CSS_DOUBLE" : "lf",
                  "CSS_DATE"   : "s",
                  "CSS_LDDATE" : "s"
                }
    def __init__ ( self,
                   name   = "CSS_LONG",
                   in_val = 0,
                   t_form = "%ld",
                 ) :
        self.name      = name
        self.in_val    = in_val
        self.t_form    = t_form

        ## check invariant
        self.__check_inv ()
    # end def __init__

    def __str__ (self) :
        e_str = "CssType: %s [init_v: %s bytes, format: \"%s\"]" \
                %( self.name,  self.in_val, self.t_form)
        return e_str
    #end def __str__

    def __check_inv (self) :
        """private: checks the css type"""
        if not CssType.css_types.has_key (self.name) :
            m = "unknown Css Type: %s" %self.name
            raise InvCssType (m)
        else :
            if self.t_form.find (CssType.css_types[self.name]) == -1:
                m = "the Css Type: %s has an invalid format: %s" \
                         %(self.name, self.t_form)
                raise InvCssType (m)
    # end def __check_inv
# end class CssType

class CssDesc :
    """ This class modulates the CssDescriptors (see gbase documentation)
    """
    def __init__ ( self,
                   name,
                   d_len  = 4,
                   d_type = CssType ()
                 ) :
        self.name  = name
        self.d_len = d_len
        self.start = 0
        self.end   = 0
        self.type  = d_type
    # end def __init__

    def __str__ (self) :
        if self.type.in_val == "T_NULL" :
            init_val = "%s" %self.type.in_val
        else:
            init_val = "\"%s\"" %self.type.in_val

        e_str = "{%d, %d, \"%s\", \"%s\", %s, 0, %s, %s}" \
                %( self.start,
                   self.end,
                   self.name,
                   self.type.t_form,
                   "offset(%s)" %self.name,
                   self.type.name,
                   init_val
                 )
        return e_str
    #end def __str__

    def calculate (self, offset) :
        """ Calculates the start and the end relative to
            'offset'
        """
        self.start = offset + 1
        self.end   = self.start + self.d_len - 1
        return self.end + 1
    #end def calculate
# end class CssDesc

class CssTable :
    """This class reads the input description file and generates the
       CssTable's offsets
             (?P<type>[a-zA-Z0-9]*_[a-zA-Z0-9]*)     # descriptors type
    """
    def __init__ ( self,
                   idf_name,
                   output_fname = None
                 ) :
        self.fname       = idf_name
        self.output      = self.__open_output (output_fname)
        self.idf         = self.__open_idf    ()
        self.linenr      = 1
        self.descriptors = []
        self.pattern     = re.compile(r"""
             \s*                        # Skip leading whitespace

             (?P<name>([a-zA-Z0-9]*_*[a-zA-Z0-9]*)|([a-zA-Z0-9]_*))     # descriptors name - ok
             ,\s*                       # Whitespace, and a comma
             (?P<size>[0-9]*)           # The size
             ,\s*                       # Whitespace, and a comma
             (?P<type>CSS_[A-Z]*)       # descriptors type
             ,\s*                       # Whitespace, and a comma
             (?P<i_val>([a-zA-Z0-9]_*[a-zA-Z0-9]*)|(-)([a-zA-Z0-9]_*)|-|[0-9]*.) # init value
             ,\s*                       # Whitespace, and a comma
             (?P<format>\%(.)[a-zA-Z0-9]*) # init value
            """, re.VERBOSE)
    # end def __init__

    def __open_output (self, output_fname) :
        """private: opens the output file. If 'output_fname' is None
           sys.stdout is used instead.
        """
        if output_fname is None :
            return sys.stdout
        else:
            try:
                f = open (output_fname, "w")
            except IOError, (errno, strerror):
                print "I/O error(%s): %s '%s'" \
                      % (errno, strerror, output_fname)
                raise FClose ()
            return f
        # end if
    # end def __open_output

    def __open_idf (self) :
        """private: open the input description file """
        try:
            f = open (self.fname, "r")
        except IOError, (errno, strerror):
            print "I/O error(%s): %s '%s'" % (errno, strerror, self.fname)
            raise FClose ()
        return f
    # end def __open_idf

    def __check_version (self) :
        """private: checks the version of the input description file """
        res  = []
        line = self.idf.readline ()

        while line != '':
            if string.find (line, "Version") >= 0:

                ## Version found => parse it
                res = string.split (line, "=")
                if len (res ) != 2:
                    em  = "Unexpected format at line #%d.\n"
                    em += "Please correct the format and try again."
                    print "%s" %em % self.linenr
                    raise FClose ()
                else:
                    expver = '%s.%s.%s' %(ffv_major, ffv_minor, ffv_patch)
                    ver    = string.rstrip (res[1], string.whitespace)

                    if ver != expver:
                        msg = "Version mismatch in the input description file"
                        msg += " found.\nExpected version: %s, found version: %s"
                        print "%s" %msg %(expver, ver)
                        raise FClose ()
                    else:
                        ## the versions matched
                        return
            ## end if

            self.linenr += 1
            line    = self.idf.readline ()
        ## end while

        ## no version is found => report an error and
        print "EOF '%s' is reached searching for the version." %self.fname
        print "Please correct the input description file and try again."
        raise FClose ()
    # end def __check_version

    def __parse_line (self, line) :
        """private: parses the line"""

        if line == '\n' or line.find ("###") == 0:
            ## this is an empty or comment line
            return
        # end if

        m = self.pattern.search (line)
        if m :
            try:
                t = CssType (m.group ('type'),
                             m.group ('i_val'),
                             m.group ('format')
                             )
            except InvCssType, exp:
                m = "Parsing failed at line #%s: %s" \
                    %(self.linenr, str (exp))
                print m
                raise FClose ()

            d = CssDesc (m.group ('name'),
                         string.atoi (m.group ('size')),
                         t
                        )
            self.descriptors.append (d)
        else:
            m  = "The line #%d of '%s' is not successfully parsed." \
                 %(self.linenr, self.fname)
            m += "\nPlease correct the line and try again."
            print m
            raise FClose ()
        # end fi
    # end def __parse_line

    def __parse_idf (self) :
        """private: parses the input description file"""

        line         = self.idf.readline ()
        self.linenr += 1

        while line != '':
            try:
                self.__parse_line (line)

            except FClose:
                self.idf.close    ()
                self.output.close ()
                raise FClose      ()

            ## increment the line number
            self.linenr += 1

            ## read the next line from the input description file
            line   = self.idf.readline ()
        # end while
    # end def __parse_idf

    def generate (self) :
        """public: calls all needed private functions to parse the idf
           file, and generates the CssTable offsets
        """
        try:
            self.__check_version ()
            self.__parse_idf     ()
        except FClose:
            pass
            return

        self.output.write ("\nThe Css Descriptors for %s are:\n" % self.fname)

        offset = 0
        for d in self.descriptors :
            offset = d.calculate (offset)
            self.output.write ("    %s,\n" %d)
        # end for

        self.output.write ("    {0, 0, \"\", \"\", 0, 0, 0, 0,  \"\"},\n")
        self.idf.close    ()
        self.output.close ()
    # end def generate
# end class CssTable

def ut_types () :
    t_list = [ CssType ("CSS_INT",           0, "%d"),
               CssType ("CSS_LONG",          0, "%ld"),
               CssType ("CSS_FLOAT",       0.0, "%f"),
               CssType ("CSS_DOUBLE",      0.0, "%lf", 4),
               CssType ("CSS_STRING",  "\"-\"", "%s")
             ]
    for t in t_list :
        print t
#end def ut_types

def ut_desc () :
    print "The wfdisc table is generated:"
    d_list = [ CssDesc ("sta", 6, CssType ("CSS_STRING",
                                           "\"-\"",
                                           "%s"
                                           )
                       ),
               CssDesc ("chan", 8, CssType ("CSS_STRING",
                                           "\"-\"",
                                           "%s"
                                           )
                       ),
               CssDesc ("time", 17, CssType ("CSS_TIME",
                                           "T_NULL",
                                           "%5lf"
                                           )
                       ),
               CssDesc ("wfid", 8, CssType ("CSS_LONG",
                                           "\"-1\"",
                                           "%ld"
                                           )
                       ),
               CssDesc ("chanid", 8, CssType ("CSS_LONG",
                                           "\"-1\"",
                                           "%ld"
                                           )
                       ),
               CssDesc ("jdate", 8, CssType ("CSS_LONG",
                                           "\"-1\"",
                                           "%ld"
                                           )
                       ),
               CssDesc ("endtime", 17, CssType ("CSS_TIME",
                                           "T_NULL",
                                           "%5lf"
                                           )
                       ),
               CssDesc ("nsamp", 8, CssType ("CSS_LONG",
                                           "\"-1\"",
                                           "%ld"
                                           )
                       ),
               CssDesc ("samprate", 11, CssType ("CSS_FLOAT",
                                           "\"-1.\"",
                                           "%7lf"
                                           )
                       )
             ]
    offset = 0
    for d in d_list :
        offset = d.calculate (offset)
        print d
#end def ut_desc

def unit_test () :
    print "CssTable: running unit tests\n"
    ut_desc  ()
#end def unit_test

def usage () :
    print "usage: CssTable.py -opt idf (desc)"
    print "Options:"
    print "       -gen  Generates the css offsets"
    print "       -ut   Executes the unit tests"
    print "       -help Prints out this screen"
    print "Arguments:"
    print "       idf   The name of the input description file"
    print "       desc  The name of the output file (optional)"
    sys.exit (1)
# end def usage

def parse_cmd_params () :
    args   = sys.argv[1:]
    option = ""

    if len (args) :
    	option = args[0]

    if option.find ("-") >= 0 :
        if option == "-help" :
            usage ()
        if option == "-ut" :
            unit_test ()
            sys.exit (0)
        else :
            if option != "-gen" :
                print "Unknown option %s\n" %option
                usage ()
            # end if
    else :
        print "Insufficient options\n"
        usage ()
    # end if

    if len (args) < 2:
        print "Insufficient arguments passed\n"
        usage ()
    # end if

    if len (args) < 3 :
        return (sys.argv [2], None)
    else :
        return (sys.argv [2], sys.argv [3])
    # end if
# end def parse_cmd_params

if __name__ == "__main__" :

    input_fname, output_fname = parse_cmd_params ()

    try:
        css_tab = CssTable (input_fname, output_fname)
        css_tab.generate ()

    except FClose:
        pass
# end if __main__

def foo (opt) :
    if opt != "-gen" and opt != "-ut" :
        print "Unknown option %s\n" %opt
    elif opt != "-ut":
        print "Insufficient arguments passed\n"
    else:
        print "Foo"
# end def foo

### __END__ CssTable
