#!/usr/bin/perl -w

#
#  Script to generate text files containing the current state
#  of a station. The current state is determined by scanning
#  the event output generated by the reporting tool
#
#  ----------------------------------------------------------
#  Author: Gerald Klinkl, PSE KB22, gerald.klinkl@siemens.com
#  Last modification $Date: 2010-05-06 13:44:56 $
#  $Revision: 1.6 $
#  ----------------------------------------------------------
#  This script uses the output from the reporting tool
#
#  Usage: currentView.pl config_file
#  ----------------------------------------------------------
#  ToDo:
#  ----------------------------------------------------------
#  $Log: currentView.pl,v $
#  Revision 1.6  2010-05-06 13:44:56  gerald
#  Replace string 'missing' with 'absent' when a channel
#  has not been received and add more function comments
#
#  Revision 1.5  2007/07/23 13:08:07  martins
#  Added the connection level related bandwidth events
#  SUFFICIENT_BW | INSUFFICIENT_BW.
#  Removed the frame level events TIMELY_IBW | BACKFILLING_IBW |
#  NO_TIMELY_DATA_IBW.
#
#  Revision 1.4  2007/06/28 07:49:03  gerald
#  Add new station state 'No timely data arrived'
#
#
#

use POSIX qw(strftime);

my $stationStates = {};
my $oldStationStates = {};
my $channelStates = {};
my $oldChannelStates = {};
my $debugLevel = 1;
my $now = 0;
my $event;
my $currViewPath;

# ------------------------------------------------------------------------
# Log debug messages if debuglevel >= debug priority
# ------------------------------------------------------------------------

sub debug()
{
   print "DEBUG:$_[0]\n" if ($debugLevel);
}

# ------------------------------------------------------------------------
# Log invalid line messages
# ------------------------------------------------------------------------

sub invalidLine()
{
   chop;
   print "Warning: Invalid event line found [$_[0]]\n";
}

# ------------------------------------------------------------------------
# Create station object which describes current state of station
# ------------------------------------------------------------------------

sub createStation()
{
   local $station = { NAME => $_[0],
                   CONNECTED => 'unknown',
                   TIME_OF_LAST_CHANGE => '0.0',
                   FORMAT => 'unknown',
                   REMOTE => '0',
                   LOCAL => '0',
                   BANDWIDTH => 'unknown',
                   TIMELY => 'unknown',
                   TIME_OF_LAST_T_CHANGE => '0.0',
                   NOMINAL_TIME => '0.0',
                   FPC => '0',
                   OLD_VALUES => '',
                   LAST_WRITE => 0
                  };

   $stationStates{$_[0]} = $station;
   &debug("New station: $_[0]");
   return($station);
}

# ------------------------------------------------------------------------
# Create channel object which describes current state of channel
# Link to a station in channelStates
# ------------------------------------------------------------------------

sub createChannel()
{
   local $channel = { NAME => $_[1],
                   PRESENT => 'unknown',
                   TIME_OF_LAST_CHANGE => '0.0',
                   TIME_OF_LAST_UPDATE => '0.0',
                   STATUS => '0',
                   EARLIEST_DATA => '0.0',
                   LATEST_DATA => '0.0',
                   CPC => '0',
                  };

   $channelStates{$_[0]}{$_[1]} = $channel;
   &debug("New channel: $_[0]/$_[1]");
   return($channel);
}

# ------------------------------------------------------------------------
# Update output file
# ------------------------------------------------------------------------

sub updateFile()
{
   my $station = $_[0];
   my ($actChannelName, %actChannel, $outTxt);

   &debug("Station name: $station->{NAME}, event: $event");
   $station->{LAST_WRITE} = $now;

   open(FH, ">$currViewPath/$station->{NAME}.cv")
        or die "cannot open station $currViewPath/$station->{NAME}.cv file ";

   $outTxt = "*** Current values ***\n";
   $outTxt .= &getContent($station);
   if ($station->{OLD_VALUES} ne '' )
   {
      $outTxt .= "*** Old values ***\n";
      $outTxt .= $station->{OLD_VALUES};
   }

   print FH $outTxt;
   close FH;
}

# ------------------------------------------------------------------------
# Get states of station in human readable form
# ------------------------------------------------------------------------

sub getContent()
{
   my $station = $_[0];
   my ($actChannelName, %actChannel, $outStr);

   $outStr = " Connection state:  $station->{CONNECTED}\n" .
             "  Time of last state change: " . 
             &getTimeAsText($station->{TIME_OF_LAST_CHANGE}) . "\n";

   if ($station->{CONNECTED} ne 'connected') { return($outStr) }

   $outStr .= "  Format: CD-$station->{FORMAT}" .
              "\n  Remote Address: " . &getAddressAsText($station->{REMOTE}) .
              "\n  Local Address: " . &getAddressAsText($station->{LOCAL}) .
              "\n  Bandwidth: " . $station->{BANDWIDTH} .
              "\n Station state:  $station->{TIMELY}" .
              "\n  Time of last state change: " . 
              &getTimeAsText($station->{TIME_OF_LAST_T_CHANGE}) . 
              "\n  Frame parsing code: 0x$station->{FPC}\n";

   if (!defined($channelStates{$station->{NAME}})) { return($outStr) }
   %actChannel = %{$channelStates{$station->{NAME}}};
   for $actChannelName (keys %actChannel)
   {
      $outStr .= " Channel state: $actChannel{$actChannelName}->{PRESENT}\n" .
                 "  Last update: " . 
         &getTimeAsText($actChannel{$actChannelName}->{TIME_OF_LAST_UPDATE}) .
         "\n  Site/Channel names: $actChannel{$actChannelName}->{NAME}\n" .
         "  Channel parsing code: $actChannel{$actChannelName}->{CPC}\n" .
         "  Status: $actChannel{$actChannelName}->{STATUS}\n" .
         "  Time of last state change: " . 
         &getTimeAsText($actChannel{$actChannelName}->{TIME_OF_LAST_CHANGE}) .
         "\n  Earliest data time: " .
         &getTimeAsText($actChannel{$actChannelName}->{EARLIEST_DATA}) .
         "\n  Latest data time: " .
         &getTimeAsText($actChannel{$actChannelName}->{LATEST_DATA}) . "\n";
   }
   return($outStr);
}

# ------------------------------------------------------------------------
# Update output file for all known stations
# ------------------------------------------------------------------------

sub checkAllStations()
{
   my $actStation;

   for $actStation ( keys %stationStates )
   {
      &updateFile($stationStates{$actStation});
   }
}

# ------------------------------------------------------------------------
# Set connected state for station
# ------------------------------------------------------------------------

sub connected()
{
   my $station = $stationStates{$_[0]};

   if (!defined ($station)) { $station = &createStation($_[0]) }
   else { $station->{OLD_VALUES} = &getContent($station) }
   $station->{CONNECTED} = 'connected';
   ($station->{TIME_OF_LAST_CHANGE}, $station->{FORMAT}, 
    $station->{LOCAL}, $station->{REMOTE} ) = split (' ', $_[1]);

   &updateFile($station);
} 

# ------------------------------------------------------------------------
# Set disconnected state for station
# ------------------------------------------------------------------------

sub disconnected()
{
   my $station = $stationStates{$_[0]};

   if (!defined ($station)) { $station = &createStation($_[0]) }
   else { $station->{OLD_VALUES} = &getContent($station) }
   $station->{CONNECTED} = 'not connected';
   $station->{LOCAL} = '0.0';
   $station->{REMOTE} = '0.0';
   $station->{BANDWIDTH} = 'unknown';
   $station->{TIME_OF_LAST_CHANGE} = $_[1];

   &updateFile($station);
} 

# ------------------------------------------------------------------------
# Change station state to 'timely', 'not timely' or 'backfilling'
# ------------------------------------------------------------------------

sub changeStationState()
{
   my $station = $stationStates{$_[0]};

   if (!defined ($station)) { $station = &createStation($_[0]) }
   else { $station->{OLD_VALUES} = &getContent($station) }
   $station->{TIMELY} = $_[2];
   ($station->{TIME_OF_LAST_T_CHANGE}, $station->{NOMINAL_TIME}, 
    $station->{FPC}) = split (' ', $_[1]);

   &updateFile($station);
} 

# ------------------------------------------------------------------------
# Change bandwidth state of station to 'sufficient' or 'insufficient'
# ------------------------------------------------------------------------

sub changeBandwidthState()
{
   my $station = $stationStates{$_[0]};

   if (!defined ($station)) { $station = &createStation($_[0]) }
   else { $station->{OLD_VALUES} = &getContent($station) }
   $station->{BANDWIDTH} = $_[2];
   $station->{TIME_OF_LAST_CHANGE} = $_[1];

   &updateFile($station);
} 

# ------------------------------------------------------------------------
# Change station state to 'timely'
# ------------------------------------------------------------------------

sub timely()
{
   &changeStationState(@_, 'timely');
}

# ------------------------------------------------------------------------
# Change station state to 'backfilling'
# ------------------------------------------------------------------------

sub backfilling()
{
   &changeStationState(@_, 'backfilling');
} 

# ------------------------------------------------------------------------
# Change bandwidth state of station to 'sufficient'
# ------------------------------------------------------------------------

sub sufficientBW()
{
   &changeBandwidthState(@_, 'sufficient');
} 

# ------------------------------------------------------------------------
# Change bandwidth state of station to 'insufficient'
# ------------------------------------------------------------------------

sub insufficientBW()
{
   &changeBandwidthState(@_, 'insufficient');
} 

# ------------------------------------------------------------------------
# Change station state to 'not timely'
# ------------------------------------------------------------------------

sub noTimelyData()
{
   &changeStationState(@_, 'no timely data');
} 

# ------------------------------------------------------------------------
# Change station state to 'no data received'
# ------------------------------------------------------------------------

sub noData()
{
   my $station = $stationStates{$_[0]};

   if (!defined ($station)) { $station = &createStation($_[0]) }
   else { $station->{OLD_VALUES} = &getContent($station) }
   $station->{TIMELY} = 'no data';
   $station->{TIME_OF_LAST_T_CHANGE} = $_[1];

   &updateFile($station);
}

# ------------------------------------------------------------------------
# Change station state to 'no data received'
# ------------------------------------------------------------------------

sub frameStatusChange() 
{
   my $station = $stationStates{$_[0]};

   if (!defined ($station)) { $station = &createStation($_[0]) }
   else { $station->{OLD_VALUES} = &getContent($station) }
   ($station->{TIME_OF_LAST_T_CHANGE}, $station->{NOMINAL_TIME}, 
    $station->{FPC}) = split (' ', $_[1]);

   if ($now - $station->{LAST_WRITE} > 60) { &updateFile($station); }
}

# ------------------------------------------------------------------------
# Change channel state to 'absent' or 'present'
# ------------------------------------------------------------------------

sub changeChannelValues()
{
   my $station = $stationStates{$_[0]};
   my ($eventTime, $channelName, $nomTime, $cpc, $channelState);

   if ($_[2] eq 'absent')
   {
      ($eventTime, $channelName) = split (' ', $_[1], 2);
      $channelState = 'NA';
      $cpc = 'NA';
   }
   else
   {
      ($eventTime, $channelName, $nomTime, $cpc, $channelState) = 
       split (' ', $_[1], 5); 
   }

   if (!defined ($station)) { $station = &createStation($_[0]) }
   else { $station->{OLD_VALUES} = &getContent($station) }

   my $channel = $channelStates{$_[0]}{$channelName};
   if (!defined ($channel)) { $channel = &createChannel($_[0], $channelName)}

   &debug("changeChannelValues: $_[1], eventTime -> $eventTime, $_[2]");
   $channel->{PRESENT} = $_[2];
   $channel->{TIME_OF_LAST_CHANGE} = $eventTime;
   $channel->{TIME_OF_LAST_UPDATE} = $now; 
   $channel->{STATUS} = $channelState;
   $channel->{STATUS} = 
              "0x$channel->{STATUS}" unless ($channel->{STATUS} eq "NA");
   $channel->{CPC} = $cpc;
   $channel->{CPC} = "0x$channel->{CPC}" unless ($channel->{CPC} eq "NA");

   if ($_[2] ne 'absent')
   {
      $channel->{EARLIEST_DATA} = $nomTime if 
                 ($channel->{EARLIEST_DATA} == 0.0);
      $channel->{EARLIEST_DATA} = $nomTime if 
                 ($channel->{EARLIEST_DATA} > $nomTime);
      $channel->{LATEST_DATA} = $nomTime if 
                 ($channel->{LATEST_DATA} < $nomTime);
   }

   if ($now - $station->{LAST_WRITE} > 60) { &updateFile($station); }
}

# ------------------------------------------------------------------------
# Change channel state to 'present'
# ------------------------------------------------------------------------

sub channelPresent()
{
   &changeChannelValues(@_, 'present');
}

# ------------------------------------------------------------------------
# Change channel state to 'absent'
# ------------------------------------------------------------------------

sub channelMissing()
{
   &changeChannelValues(@_, 'absent');
}

sub channelStatusChange()
{
   &changeChannelValues(@_, 'present');
}

# ------------------------------------------------------------------------
# Info Event; update channel statistics
# ------------------------------------------------------------------------

sub info() 
{
   my $station = $stationStates{$_[0]};
   my ($eventTime, $channelName, $earliest, $latest, $current) = 
          split (' ', $_[1], 5);

   if (!defined ($station)) { $station = &createStation($_[0]) }
   else { $station->{OLD_VALUES} = &getContent($station) }

   my $channel = $channelStates{$_[0]}{$channelName};
   if (!defined ($channel)) { $channel = &createChannel($_[0], $channelName)}

   &debug("eventTime -> $eventTime");
   &debug("current -> $current");
   $channel->{TIME_OF_LAST_CHANGE} = $eventTime;
   $channel->{TIME_OF_LAST_UPDATE} = $now;
   $channel->{EARLIEST_DATA} = $earliest;
   $channel->{LATEST_DATA} = $latest;

   if ($now - $station->{LAST_WRITE} > 60) { &updateFile($station); }
}

my %eventTab = ( k => \&connected, l => \&disconnected,
                 t => \&timely, b => \&backfilling,
                 a => \&sufficientBW, A => \&insufficientBW,
                 w => \&frameStatusChange,
                 c => \&channelPresent, C => \&channelMissing,
                 s => \&channelStatusChange,
                 i => \&info, N => \&noData,
                 o => \&noTimelyData );

# ------------------------------------------------------------------------
# Convert IP address into human readable form
# ------------------------------------------------------------------------

sub getAddressAsText()
{
   my ($IP, $port) = split ('\.', $_[0], 2);
   my $txtIP;

   $txtIP = ($IP & 0xFF000000) >> 24;
   $txtIP .= sprintf ".%d", ($IP & 0x00FF0000) >> 16;
   $txtIP .= sprintf ".%d", ($IP & 0x0000FF00) >> 8;
   $txtIP .= sprintf ".%d", $IP & 0x000000FF;
   $txtIP .= sprintf ":%d", $port;
   return ($txtIP);
}

# ------------------------------------------------------------------------
# Convert time into human readable form
# ------------------------------------------------------------------------

sub getTimeAsText()
{
   my ($seconds, $mSeconds) = split ('\.', $_[0], 2);
   $mSeconds = 0 unless defined($mSeconds); 
   my $txt = strftime("%Y/%m/%d %H:%M:%S", gmtime($seconds));
   return("$txt.$mSeconds");
}

if (@ARGV != 1) { die "usage: $0 config_file \n"; }

# ------------------------------------------------------------------------
# Read the configuration file
# ------------------------------------------------------------------------

my $configFile = shift(@ARGV);
$currViewPath = `egrep '^currViewPath=' $configFile`;
$currViewPath = './' unless ($currViewPath ne '');
&debug("currViewPath [$currViewPath]");
$currViewPath =~ s/.*=//;


# ------------------------------------------------------------------------
# Read event lines form stdin
# ------------------------------------------------------------------------

while (<STDIN>)
{
   # read one line from stdin
   my ($rol, $stationName);

   chop;
   &debug("READ [$_]");

   # Split line into station name, event and rest of line
   ($stationName, $event, $rol) = split (' ', $_, 3);
   $now = time;
   
   if ( !defined ($rol) ) { &invalidLine($_) }
   # Call appropriate function as defined in the event table
   if ( $eventTab{$event} ) { $eventTab{$event}->($stationName, $rol) } 
   else { &invalidLine($_) }

# if ($event eq 'l') { sleep(1) }
}

&checkAllStations();
