/*
 * FILE NAME  idcstreamlog_core.h
 *
 *
 * SYNOPSIS
 *      Prototypes and constants for the libidcstreamlog.a clients.
 *
 * AUTHORS
 *
 *      Martin Ertl, Jan Kianicka (2015-12-01)
 *
 * MODIFICATIONS
 *
 */

#ifndef IDCSTREAMLOG_P_H
#define IDCSTREAMLOG_P_H

/*
 * Preferred order of includes,
 * according to the Google Style Guide:
 *
 *      1. Our related public .h file we are amending (optional).
 *      2. Private .h file we are extending (if any).
 * 	3. C system files.
 * 	4. Other libraries' .h files.
 * 	5. Our project's .h files.
 *
 * Order files alphabetically within each section.
 */

#include <stddef.h>           	     /* size_t */
#include <stdio.h>                   /* FILE */

/*** Macro definitions ******************************************************/

#define loglog (*idcstreamlog_core_get_log())
#define sizeof_(str, elem)    sizeof(((str *)0)->elem)

/**
 * Macro switched debug logging with variable arguments.
 *
 * Usage:
 *   FPRINTF((L_DBG, T_GEN, "Hello debug!\n"));
 *   FPRINTF((L_DG1, T_RUN, "var1=%d, var2=%d\n", var1, var2));
 *
 * Build with CFLAGS='-DDEBUG_IDCSTREAMLOG=n':
 *   n=0: disables debug macros
 *   n>0: enables debug macros
 *
 * DEBUG_IDCSTREAMLOG (if set) also controls the default log upto:
 *  default_log_upto = IDCSTREAMLOG_S_WRN + DEBUG_IDCSTREAMLOG
 */
#ifdef FPRINTF
#undef FPRINTF
#undef VFPRINTF
#endif

#if DEBUG_IDCSTREAMLOG > 0
#define FPRINTF(args) fprintf args
#define VFPRINTF(stream, fmt, ap) vfprintf(stream, fmt, ap)
#else
#define FPRINTF(args)
#define VFPRINTF(args)
#endif

/*** Type definitions *******************************************************/

/**
 * Element attributes.
 */
typedef struct idcstreamlog_elemattr_
{
    char     *name;
    char     *pattern;
    size_t   offset;
    int      size;
} idcstreamlog_elemattr_t;

/**
 * Message severity.
 */
typedef enum idcstreamlog_severity_
{
    IDCSTREAMLOG_S_EMG = 0,  /* syslog LOG_EMERG                   */
    IDCSTREAMLOG_S_ART = 1,  /* syslog LOG_ALERT                   */
    IDCSTREAMLOG_S_CRT = 2,  /* syslog LOG_CRIT                    */
    IDCSTREAMLOG_S_ERR = 3,  /* syslog LOG_ERR                     */
    IDCSTREAMLOG_S_WRN = 4,  /* syslog LOG_WARNING                 */
    IDCSTREAMLOG_S_NTC = 5,  /* syslog LOG_NOTICE                  */
    IDCSTREAMLOG_S_IFO = 6,  /* syslog LOG_INFO                    */
    IDCSTREAMLOG_S_DBG = 7,  /* syslog LOG_DEBUG = 7               */
    IDCSTREAMLOG_S_DG1 = 8,  /* ditto                              */
    IDCSTREAMLOG_S_DG2 = 9,  /* ditto                              */
    IDCSTREAMLOG_S_DG3 = 10, /* ditto                              */
    IDCSTREAMLOG_S_OFF = 11, /* for verbosity threshold value only */
    IDCSTREAMLOG_S_MIN = 0,
    IDCSTREAMLOG_S_MAX = 11,
    IDCSTREAMLOG_S_CNT = IDCSTREAMLOG_S_MAX - IDCSTREAMLOG_S_MIN + 1
} idcstreamlog_severity_t;

/**
 * Standard streams.
 */
typedef enum idcstreamlog_stdstream_
{
    IDCSTREAMLOG_STD_ERR = 0,
    IDCSTREAMLOG_STD_OUT = 1,
    IDCSTREAMLOG_STD_OFF = 2,
    IDCSTREAMLOG_STD_MIN = 0,
    IDCSTREAMLOG_STD_MAX = 2,
    IDCSTREAMLOG_STD_CNT = IDCSTREAMLOG_STD_MAX - IDCSTREAMLOG_STD_MIN + 1
} idcstreamlog_stdstream_t;

/*** Prototypes *************************************************************/

/* Interface function declarations */

/**
 * Accessor/setter functions.
 */
idcstreamlog_severity_t idcstreamlog_core_get_severity(char const *name);
FILE *idcstreamlog_core_get_errstream();
FILE *idcstreamlog_core_get_outstream();
FILE *idcstreamlog_core_get_offstream();
struct idcstreamlog_systopic_ *idcstreamlog_core_get_log();

#endif
