#ifndef _COMMON_CD_H
#define _COMMON_CD_H

/**
 * @author    Gerald Klinkl
 *
 * Revision 1.97  2014-08-19 gerald
 * Move crl pointer to certificate cache to support more
 * than one CRL
 *
 * Revision 1.96  2014-02-05 GK
 * Update member comment
 *
 * Revision 1.95  2013-06-25 gerald
 * Add function which cleans up library (necessary for OpenSSL
 * engine support)
 *
 * Revision 1.94  2013-04-03 gerald
 * Support new idcsyslog public header filename (from cbase 1.1.0)
 *
 */

#ifdef S_SPLINT_S
 #include "splint_hints.h"
#endif /* S_SPLINT_S */

#include <stdio.h>
#include <pthread.h>
#include <signal.h>

#ifdef HAVE_AUTHENTICATION
 #include "auth.h"
 #ifdef HAVE_LDAP
  #include <lber.h>
  #include <ldap.h>
 #endif /* ifdef HAVE_LDAP */
#endif /* ifdef HAVE_AUTHENTICATION */

#include "libpar.h"
#include "misc.h"
#include "frames.h"

/* ==== File name extensions and macros  ==================================== */

#define FILE_NAME_SIZE     1024   /* Max length of file path plus name */
#define DATE_FORMAT        "%Y%m%d.%H%M"    /* strftime format */
#define DATE_FORMAT_SCAN   "%4d%2d%2d.%2d%2d"    /* scanf format */
#define DATE_FORMAT_IDX    "%Y%m%d.idx"    /* strftime format */
#define LEN_STATION_NAME   8     /* Max. len of station name */
#define LEN_CON_ID        17     /* Max. len of connection ID. For cdsender
                                    ID is DPname_DCname, for cdreceiver
                                    ID is station name */
#define LEN_DATE_PART     16     /* Max. len of date part in file name */
#define MAX_FILE_NAME   (LEN_DATE_PART + LEN_CON_ID + 2 + 10)

#define MIN_BLOCKS_FREE  2000   /* Minimum number of blocks which have to
                                   be available on the filePath file system
                                   that the application will start */

#define EXTENSION_BIN_CD1     "10bin"
#define EXTENSION_BIN_CD11    "11bin"
#define EXTENSION_OFFLINE     "_off"
#define EXTENSION_IDX         "idx"
#define EXTENSION_LST         "clf"
#define EXTENSION_IRL         "irl"

#define FILE_NAME_FORMAT "%.17s.%.16s.%.10s%.5s"
#define BUILD_FILE_NAME(buffer, bufSize, conID, dateTime, extension, olExt) \
      (void) snprintf((buffer), (bufSize), "%.17s.%.16s.%.10s%.5s", (conID), \
                      (dateTime), (extension), (olExt))
#define ADD_ERRORS(pStation) \
        ((unsigned int)((pStation)->isConnected + \
                        (pStation)->invalidFrames + \
                        (pStation)->invalidStations + \
                        (pStation)->invalidIPs + \
                        (pStation)->timeouts + \
                        (pStation)->authenticationFailures + \
                        (pStation)->closedConnections + \
                        (pStation)->systemFailures))
#define ADD_CMD_FRAMES(pReceived) \
        ((unsigned int)((pReceived)->commandRequest + \
                        (pReceived)->commandResponse))
#define ADD_FRAMES(pReceived) \
        ((unsigned int)((pReceived)->connectionRequest + \
                        (pReceived)->connectionResponse + \
                        (pReceived)->dataFormat + \
                        (pReceived)->data + \
                        (pReceived)->alert + \
                        (pReceived)->ackNak + \
                        (pReceived)->optionRequest + \
                        (pReceived)->optionResponse + \
                        (pReceived)->commandRequest + \
                        (pReceived)->commandResponse + \
                        (pReceived)->invalid))


#define DIR_PERMS   0777  /* Directory permissions before umask() is applied */
#define FILE_PERMS  0666  /* File permissions before umask() is applied */

#define DEBUG_FP  stderr
#define S_DEBUG_HDR    (void)fprintf(DEBUG_FP, "(SDebug:%.2hu)[%.17s] ", \
                                     pStation->debugLevel, pStation->conID)
#define IF_ST_DEBUG(level)        if (pStation->debugLevel >= level)
#define ST_DEBUG(level,msg)       IF_ST_DEBUG(level)\
        { S_DEBUG_HDR;\
          (void)fprintf(DEBUG_FP, "%s\n", msg); (void)fflush(DEBUG_FP);}
#define ST_DEBUG2(level,fmt,p1)   IF_ST_DEBUG(level)\
        { S_DEBUG_HDR;\
          (void)fprintf(DEBUG_FP, fmt, p1); (void)fprintf(DEBUG_FP, "\n");\
          (void)fflush(DEBUG_FP);}
#define ST_DEBUG3(level,fmt,p1, p2)  IF_ST_DEBUG(level)\
        { S_DEBUG_HDR;\
          (void)fprintf(DEBUG_FP, fmt, p1, p2);\
          (void)fprintf(DEBUG_FP, "\n");\
          (void)fflush(DEBUG_FP);}
#define ST_DEBUG4(level,fmt,p1, p2, p3)  IF_ST_DEBUG(level)\
        { S_DEBUG_HDR;\
          (void)fprintf(DEBUG_FP, fmt, p1, p2, p3);\
          (void)fprintf(DEBUG_FP, "\n");\
          (void)fflush(DEBUG_FP);}
#define DEBUG_FOOTER   (void)fflush(DEBUG_FP);

#define MAX_CONNECTIONS  150    /* Max. number of concurrent connections */

/* ==== Defines and data structures for file info =========================== */

typedef struct _fileDes {
	time_t   savedDay;   /**< Day since start of Epoch when the last
                                file has been created */
        char     savedFileName[LEN_DATE_PART];
#define LEN_WFDISC_DIR   65     /**< Max. len of file path in wfdisc record */
        char     savedFilePath[LEN_WFDISC_DIR];
        unsigned char  lastType;   /**< protocol type when file was created */
        /*@null@*//*@only@*/
	FILE     *fpBinFile;   /**< file pointer bin file */
	uint32_t sizeBinFile;  /**< accumulated size bin file */
        /*@null@*//*@only@*/
	FILE     *fpTxtFile;  /**< file pointer text file */
        /*@null@*//*@only@*/
	FILE     *fpWfdiscFile;  /**< file pointer wave disc records file */
        /*@null@*//*@only@*/
	FILE     *fpClfFile;  /**< file pointer clf file */
        /*@null@*//*@only@*/
	FILE     *fpUsrFile;  /**< file pointer user defined file */
        /*@null@*//*@only@*/
	FILE     *fpAppFile;  /**< file pointer application specific file */
	uint32_t clfRecNum;  /**< frame number (only CD 1.0) */
} fileDes_t;

/* ==== Defines and data structures for port map ============================ */

/**
 * Port mapping structure
 *
 * This structure is used to track port utilization and port information
 */
typedef struct _portMap {
   uint16_t localPort;          /**< Local port used */
   char     localIPaddr[16];    /**< Local IP address */
   uint16_t remotePort;         /**< Remote port used */
   char     remoteIPaddr[16];   /**< Remote IP address */
   int      socket;             /**< Socket descriptor */
} portMap_t;

/* ==== Defines and data structure for station IP address  ================== */

/**
 * Station IP structure
 *
 * This structure is used to check if a connection request is allowed and
 * to maintain network address transformation information
 */
typedef struct _stationIP {
   char      authName[LEN_STATION_NAME + 1];  /**< station name or '*' */
   in_addr_t authIpAddress;                   /**< IP address or 0 (=ANY)  */
#define LEN_IP_STR  16
   char      redirectIP[LEN_IP_STR];          /**< empty or IP send back in
                                                   port assignment process */
#ifdef HAVE_AUTHENTICATION
   /*@dependent@*//*@null@*/
   keyCache_t *keyCacheSta;       /**< key cache for forwarder keys
                                       and certificates */
#endif /* endif HAVE_AUTHENTICATION */
} stationIP_t;

/* ==== Defines and data structures for statistic data  ===================== */

/**
 * Frame statistics structure. Used to store frame related statistics
 *
 * This structure is used to store number of received or sent
 * frames divided by frame type
 */
typedef struct _CDframeStatistics
{
   uint32_t connectionRequest; /**< connection request frames */
   uint32_t connectionResponse; /**< connection response frames */
   uint32_t dataFormat; /**< data format frames */
   uint32_t data; /**< data frames */
   uint32_t alert; /**< alert frames */
   uint32_t ackNak; /**< ackNack frames */
   uint32_t optionRequest; /**< option request frames */
   uint32_t optionResponse; /**< option response frames */
   uint32_t commandRequest; /**< command request frames */
   uint32_t commandResponse;  /**< command response frames */
   uint32_t invalid;  /**< invalid frames */
   uint32_t bytesTotal; /**< bytes total in connected state */
   uint32_t bytesTotalCR; /**< bytes total in unconnected state */
} CDframeStatistics_t;

/**
 * Error structure. Used to store errors or warnings
 *
 * This structure is used to store errors or warnings during
 * connection requests or when connected. Errors are counted by error type
 */
typedef struct _CDerrorStatistics
{
   uint32_t warnings;    /**< Number of warnings */
   uint32_t isConnected; /**< station already connected (only during CR)/time
                              based sync */
   uint32_t timeouts;    /**< station timed out */
   uint32_t invalidIPs;  /**< connect() from unauthorized IP address */
   uint32_t invalidFrames;  /**< invalid frame */
   uint32_t invalidStations;  /**< unauthorized station name / frame based
                                   sync */
   uint32_t authenticationFailures;  /**< frame/channel authentication failed */
   uint32_t closedConnections;  /**< connection closed unexpected */
   uint32_t systemFailures;   /**< system (call) failure */
   uint32_t connectsOK; /**< Successful (TCP) connects/ DB commit (cd2wng) */
} CDerrorStatistics_t;

/* ==== Defines and data structures for station ============================= */

/**
 * Station structure. Used to store station specific data
 *
 * This structure is used to store and remember station specific data.
 * Station structures are managed as a linked list
 */
typedef struct _station {
   char stationName[LEN_STATION_NAME+1]; /**< station name */
   char conID[LEN_CON_ID+1];       /**< connection ID */
   unsigned char  state;           /**< connection states, see below */
#define         INTERNAL_FAILURE        ((unsigned char)0)
#define         STATION_UNCONNECTED     ((unsigned char)1)
#define         STATION_CONNECTING      ((unsigned char)2)
#define         STATION_WAITING         ((unsigned char)3)
#define         STATION_CONNECTED       ((unsigned char)4)
#define         STATION_WAIT2SEND       ((unsigned char)5)
#define         STATION_STOPPED         ((unsigned char)6)
   unsigned char  type;        /**< CD protocol types, see below */
/* INTERNAL_FAILURE is 0 */
#define         UNKNOWN        ((unsigned char)1)
#define         CD1            ((unsigned char)2)
#define         CD1_1          ((unsigned char)3)

#ifdef HAVE_AUTHENTICATION
   /*@dependent@*/
   keyCache_t *keyCache;
#endif /* endif HAVE_AUTHENTICATION */

   /*@shared@*/
   portMap_t *portInfo;           /**< current port information */
   /*@null@*//*@dependent@*/
   const stationIP_t *pStationIP;  /**< station file info */
   pthread_t threadID;
   char *frameBuffer;              /**< input buffer */
   uint32_t frameBufferSize;       /**< current input buffer size */
   /*@null@*/
   char *formatBuffer;             /**< data format frame buffer */
   uint32_t formatBufferSize;      /**< current data format frame buffer size */
/* Next 3 members are used by CD receiver and the sender API */
   uint32_t lenOfDFF;              /**< size of last DFF received */
   uint16_t eventDFF;              /**< event of last DFF received */
   char   clfTypeDFF;              /**< clf type of last DFF received */
   unsigned char shutdownEnable;   /**< shutdown mode: SD_FAST or SD_NORMAL */
   CDheader_t header;              /**< current frame info; not all fields
                                        are filled when CD1 station */
   uint32_t systemConf;            /**< station specific configuration value,
                                        see cd_common.h */
   unsigned short debugLevel;      /**< station specific debug level */
   uint16_t authenSummary;         /**< authentication summary frame and/
                                        or channels */
   uint16_t authenPassFrame;       /**< frame signature state */
   uint32_t checkPass;             /**< frame parsing code */
   uint32_t savedSystemErr;        /**< last system error logged */
   /*@null@*/
   CDdata_t *DFinfo;               /**< current data frame information */
   /*@dependent@*/
   const struct _configParams *pConfigData;  /**< pointer to configuration */
   fileDes_t files;
   pthread_mutex_t stationMutex;  /**< Mutex to protect station structures
                                       currently used only by cd-sender */
   time_t   lastCRFreceived; /**< time last connection request frame received */
   time_t   lastSuccessfulCR; /**< time last successful connection request */
   time_t   lastUnsuccessfulCR;   /**< time last failed connection request */
   struct timeval startRoW;    /**< time reading/writing first byte of frame */
   struct timeval endRoW;      /**< time end receiving/sending current frame */
   struct timeval lastFrameSeen; /**< time last frame of interest seen
                                      (Command or Data frame) */
   struct _CDframeStatistics framesReceived;  /**< statistic data for
                                                   received frames */
   struct _CDframeStatistics framesSent;   /**< statistic data for
                                                sent frames */
   struct _CDerrorStatistics whenUnconnected; /**< errors when unconnected */
   struct _CDerrorStatistics whenConnected;  /**< errors when connected */

   chNameCache_t chCache;        /**< channel name cache */

   /*@null@*/
   void *appExtension;     /**< container for application specific data */
   /*@owned@*/
   struct   _station *nextStation;  /**< pointer to next station structure */
} station_t;

/* ==== Defines and data structure for history information ================== */

typedef struct _history {
   time_t   appStarted;     /**< application start time */
   char     appName[10];    /**< string containing application name */
   int      appID;          /**< application ID, see below */
#define RECEIVER        0
#define SENDER          1
#define CONTROLLER      2
#define CD2W            3   /* Don't change the values for the defines or the */
#define OTHER           4   /* monitor thread will log unnecessary values */
   uint32_t nbrOfThreadsStarted;
   uint32_t nbrOfThreadsAktive;
   uint32_t nbrOfThreadsStopped;
   uint16_t nbrOfHistoryEntries;
   char     listenOn[21];            /**< listeners IPaddress and Port */
   /*@owned@*/
   struct   _station *firstStation;  /**< pointer to first station structure */
} history_t;

/* ==== Defines and data structure for system configuration information ===== */

/* Don't use #ifdef for members above or position is
   different in cd_common.c and receiver/sender/control sources */

typedef struct _configParams {
   uint32_t  systemConf;    /* system switches, see below */
#define ONLINE_MODE           (uint32_t)(1<<0)   /**< online mode (receiver) */
#define CREATE_USR_FILE       (uint32_t)(1<<1) /**< create user defined file */
#define CREATE_BIN_FILE       (uint32_t)(1<<2)   /**< create bin file */
#define CREATE_WFDISC_FILE    (uint32_t)(1<<3)   /**< create wfdisc files */
#define CREATE_TEXT_FILE      (uint32_t)(1<<4)   /**< create text summary
                                                      files */
#define CREATE_CLF_FILE       (uint32_t)(1<<5)   /**< create clf files */
#define VERBOSE_LOGGING       (uint32_t)(1<<6)   /**< log a few events
                                                      to syslog */
#define LOG_INVALID_CR        (uint32_t)(1<<7)   /**< log invalid connection
                                                      requests (receiver) */
#define LOG_VALID_CR          (uint32_t)(1<<8)   /**< log valid connection
                                                      requests (receiver) */
#define MORE_VERBOSE_LOGGING  (uint32_t)(1<<9)   /**< log all events
                                                      to syslog */
#define LOG_OUTGOING2BIN      (uint32_t)(1<<10)  /**< log outgoing frames in
                                                      bin file */
#define STRICT_MODE_ENABLE    (uint32_t)(1<<11)  /**< drop connection if frame
                                                      deviate from protocol */
#define CREATE_APP_FILE       (uint32_t)(1<<12) /**< create application
                                                     specific file */
#define FRAME_AUTH            (uint32_t)(1<<13)  /**< verify frame
                                                      authentication */
#define CHANNEL_AUTH          (uint32_t)(1<<14)  /**< verify channel
                                                      authentication */
#define ABS_WFDISC_PATHS      (uint32_t)(1<<15)  /**< use absolute file path in
                                                      wfdisc records
                                                      (receiver) */
#define MORE_TEXT_FILE        (uint32_t)(1<<16)  /**< log also channel info
                                                      in text summary file */
#define FULL_TEXT_FILE        (uint32_t)(1<<17)  /**< add description for codes
                                                      to text file */
#define READ_GROUP_PRIV       (uint32_t)(1<<18)  /**< set read permissions
                                                      for the users group
                                                      at created files */
#define WRITE_GROUP_PRIV      (uint32_t)(1<<19)  /**< set write permissions
                                                      for the users group
                                                      at created files */
#define READ_WORLD_PRIV       (uint32_t)(1<<20)  /**< set read permissions
                                                      for other users
                                                      at created files */
#define WRITE_WORLD_PRIV      (uint32_t)(1<<21)  /**< set write permissions
                                                      for other users
                                                      at created files */
#define SND_API_START_00      (uint32_t)(1<<22)  /**< files created by sender
                                                      API get midnight time;
                                                      needed by message API
                                                      to avoid too many files */
                  /* reserved (uint32_t)(1<<23) */
#define CREATE_INDEX_FILE     (uint32_t)(1<<24)  /**< create index file */
#define ENABLE_SYSLOG         (uint32_t)(1<<25)  /**< enable logging to
                                                      syslog */

   char      fileprefix[LIBPAR_BUFSIZ];    /**< File path for output files */
   char      indexFilePath[LIBPAR_BUFSIZ]; /**< File path for index file */
   char      indexFilePN[LIBPAR_BUFSIZ]; /**< File path and name for
                                                index file */
   uint32_t  fileSize;    /**< max file size, 4 Gb max. */
   uint16_t  portNumber;  /**< well known (listener) port */
   char      authStationNameFile[LIBPAR_BUFSIZ]; /**< name of the file
                          containing allowed station names and IP addresses */

   int32_t   timeOut;     /**< line idle time out CD-1.0, 24.8 days max. */
   int32_t   timeOut11;   /**< line idle time out CD1.1 */
   int32_t   CRtimeOut;   /**< connection request time out */
   int32_t   heartBeat;   /**< heart beat time */
   uint16_t  maxDeviation;  /**< max. positive deviation */

   char      keyDirectory[LIBPAR_BUFSIZ];   /**< Directory for ARM keys */
   char      certDirectory[LIBPAR_BUFSIZ];  /**< Directory for certificates */
   char      keyPreloadFile[LIBPAR_BUFSIZ]; /**< key pre-load file name */
   char      privateKeyName[LIBPAR_BUFSIZ]; /**< private key file name */
   char      secLevels[3][LIBPAR_BUFSIZ];   /**< definition for security
                                                 control levels */
   uint32_t  privateKeyID;     /**< private key ID */
   char      LDAProot[LIBPAR_BUFSIZ]; /**< LDAP server */
   char      LDAPserver[LIBPAR_BUFSIZ]; /**< DNS names(s) of LDAP server(s) */
   char      frameCreator[LIBPAR_BUFSIZ]; /**< frame creator name for CD1.1
                                               frame header */
   char      responderType[LIBPAR_BUFSIZ]; /**< frame responder type for
                                                connection req/resp frames */
   int32_t   LDAPtimeout;   /**< LDAP time out */
   uint32_t  CRLcheckTime;  /**< CRL check time */
   uint32_t  summaryRefreshTime;  /**< syslog summary refresh time */
   uint16_t  appControlPort;  /**< control port for applications
                                   (not receiver) */

   /* ------------------ not read from config file ---------------------------*/

   /*@shared@*/
   history_t *pHistory;  /**< pointer to history and station structures */
   /*@null@*/
   stationIP_t *authStationIPs; /**< list of stations allowed to connect */
   uint16_t  nbrOfAuthStations; /**< number of stations allowed to connect */
#ifdef HAVE_AUTHENTICATION
#ifdef HAVE_LDAP
   /*@null@*/
   LDAP      *ldapID;           /**< LDAP ID */
#endif /* ifdef HAVE_LDAP */
   /*@owned@*//*@null@*/
   EVP_PKEY *privateKey;        /**< pointer to private key */
   /*@owned@*//*@null@*/
   keyCache_t *CAkeyCache;   /**< key cache for CA signer certificates */
   /*@owned@*//*@null@*/
   keyCache_t *pubKeyCache;  /**< key cache for public keys and certificates */
#endif /* ifdef HAVE_AUTHENTICATION */
   unsigned char shutdownEnable;    /**< set if a NORMAL or FAST user shutdown
                                         request is pending */
#define SD_NONE       ((unsigned char) 0) /**< no shutdown pending*/
#define SD_NORMAL     ((unsigned char) 1) /**< normal shutdown */
#define SD_FAST       ((unsigned char) 2) /**< fast shutdown */
#define SD_STOP       ((unsigned char) 3) /**< stop request (similar normal) */
   uint16_t dirHint;                /**< offset in filePath where directory
                                       creation should be started */
   /*@observer@*/
   const char *offlineExt;          /**< suffix for offline mode */
   pthread_t mainID;            /**< thread ID main thread */
   pthread_mutex_t stopMutex;    /**< Mutex to signaling shutdown to Monitor */
   pthread_cond_t stopCondition; /**< Condition to signaling shutdown to
                                      monitor thread */
   pthread_mutex_t shutdownMutex;  /**< Mutex for shutdown application */
   pthread_mutex_t appMutex;   /**< Mutex which can be used by application.
                                    Currently used only by cd2w */

   /*@shared@*/
   void *configExtension; /**< Application specific extension to config data */
   size_t sizeOfAppExtension;     /**< size of appSpecific object in station
                                       structure */
   /*@null@*/
   const char *usrFileExt;  /**< suffix for user defined file */
   /*@null@*/
   const char *appFileExt;  /**< suffix for application defined file */
   /*@null@*/
   int (*appSpecificCtrl)(int cc,  /*@null@*/char *pa,
                          /*@null@*/ char *sn, char *rb, int rz);
                       /**< pointer to application specific command function */
   /*@null@*/
   int (*appSpecificMon)(station_t *pStation); /**< pointer to application
                                                   specific monitor function */
   int isInit;         /**< initialisation state of application */
#define      CONFIG_UNINITIALIZED   0
#define      CONFIG_INITIALIZED     1   /**< Basic configuration done,
                                             see initConfigParams()  */
#define      CONFIG_LOADED          2
} configParams_t;

#define CREATE_ANY_FRAME_FILE   (CREATE_BIN_FILE | CREATE_CLF_FILE | \
                                 CREATE_TEXT_FILE | CREATE_WFDISC_FILE | \
                                 CREATE_USR_FILE)

/* ==== Defines and data structure for event information ================== */

/**
 * This structure is used to add additional information to an event
 */
typedef struct _CDevent
{
   /*@dependent@*/
   char *conID;            /**< connection ID */
   short is_CR;            /**< flag if in connected state */
   unsigned char  type;    /**< event types, see below */
   /*@dependent@*/
   struct _portMap *portInfo;    /**< IP Port information */
   time_t lastCRFreceived;       /**< time last connection request received */
} CDevent_t;

/* ==== Function prototypes ================================================= */

extern void setUserMask(uint32_t systemConf);
extern void stopProgram(const char *appName, const char *msg,
                        /*@null@*/ const char *msg2);
extern int checkDirectoryPathAndSpace(const char *filePath,
                           /*@out@*/uint16_t *lenOfStaticPart,
                           const char *appName);
/*@shared@*/ const char *getStationState(unsigned char state);
/*@null@*//*@dependent@*/ extern station_t *getFirstStation();
/*@null@*//*@dependent@*/
extern station_t *getStation(const char *conID, const char *stationName);
/*@null@*//*@dependent@*/
extern station_t *checkStation(const char *stationName);
/*@dependent@*/
extern configParams_t *initConfigParams(const char *appName, const int appID);
/*@dependent@*/ extern configParams_t *getConfigPointer();
extern int getAppID();

extern void readConfigurationFile (const char *appName, const char *fileName);
extern void dumpConfigurationParameter (void);
extern void decodeSystemConf (uint32_t systemConf);
extern void shutdownApplication(const char *msg1, /*@null@*/const char *msg2);
extern void setShutdownFlag(unsigned char shutdownMode);
extern int setPortInfo(portMap_t *pPortInfo, const int newSock,
                       /*@null@*/const struct sockaddr_in *from);

#ifdef HAVE_AUTHENTICATION
extern int loadCertificates(station_t *pStation, uint32_t verbosityLevel);
extern int checkFrameAuthentication(station_t *pStation, keyCache_t *keyCache,
                         /*@null@*/ keyCache_t *pKeyCacheRe,
                         uint32_t strictMode, char *buffer, int bytesRead,
                         CDtrailer_t *trailer, uint16_t *event);
#endif /* ifdef HAVE_AUTHENTICATION */

extern uint16_t sendFrame(station_t *pStation, char *buffer, int len,
                     uint16_t posEvent, const char clfType, int isCon);
extern void cleanUp(station_t *pStation);
extern void joinHandlingThreads(station_t *firstStation);
extern /*@null@*//*@dependent@*/
station_t *getListenerObject(uint16_t portNumber);
extern /*@null@*//*@dependent@*/ const stationIP_t *
ipAddressIsKnown(const configParams_t *pConfigData, in_addr_t IPaddr);
extern /*@null@*//*@dependent@*/ const stationIP_t *
stationNameMatchIP(const configParams_t *pConfigData,
                   const stationIP_t *pStationIP, const char *stationName);
extern int adjustBufferSize(station_t *pStation, uint32_t newSize,
                            uint16_t *event);
extern uint16_t verifyChannelData(station_t *pStation);
extern void retryWait(station_t *pStation, int WaitTime);
extern /*@observer@*/ const char *CDlibVersion (void);
extern /*@observer@*/ const char *getCompileOptions (int whichOption);
extern int connectTo(station_t *pStation, rwFlags_t *flags,
                     CDevent_t *actEvent, const char *remoteIP,
                     uint16_t portNumber);
extern int wait4shutdown(sigset_t *sigMask);
extern void closeLibCD(void);
#endif /* _COMMON_CD_H */
