#ifndef _FRAMES_H
#define _FRAMES_H

/**
 * @author    Gerald Klinkl
 *
 * Revision 1.73  2014-04-28 GK
 * Update member comment
 *
 * Revision 1.72  2014-02-20 GK
 * Add function to check header of stored (not received)
 * binary files and add ER_NOT_PARSED to DF errors
 *
 * Revision 1.71  2013-08-19 gerald
 * Allow channels with a one character channel name in non
 * strict mode. See comment in getChMaskPosition() and ISI-173
 *
 * Revision 1.70  2013-05-27 gerald
 * Add station pointer to channel name cache to support
 * virtual channel name frames
 *
 * Revision 1.69  2013-03-25 gerald
 * Improve comment
 *
 */

#include <arpa/inet.h>  /* to get typedefs for stdint types */

/* ==== Defines and data structure for frames =============================== */

#define  NO_FRAME_TYPE          0
#define  CONNECTION_REQUEST     1 /**< CD1.1 Connection Request frame type */
#define  CONNECTION_RESPONSE    2 /**< CD1.1 Connection Response frame type */
#define  OPTION_REQUEST         3 /**< CD1.1 Option Request frame type */
#define  OPTION_RESPONSE        4 /**< CD1.1 Option Response frame type */
#define  DATA                   5 /**< CD1.1 Data frame type */
#define  ACK_NACK               6 /**< CD1.1 Ack_nack frame type */
#define  ALERT                  7 /**< CD1.1 Alert frame type */
#define  COMMAND_REQUEST        8 /**< CD1.1 Command Request frame type */
#define  COMMAND_RESPONSE       9 /**< CD1.1 Command Response frame type */
#define  CD1ENCAP              13 /**< CD1.1 Encapsulate frame type */

#define SHIFT_CD1             100 /**< used to process CD1 and CD1.1 in
                                       the same function. Has to be higher
                                       than the highest CD1.1 frame type */

#define  CD1_DATA               1 /**< CD1.0 Data frame type */
#define  CD1_DATA_FORMAT        2 /**< CD1.0 Data Format frame type */
#define  CD1_ALERT              4 /**< CD1.0 Alert frame type */

#define MAX_FRAME_LEN         10000000    /**< maximal frame length */
#define MIN_FRAME_LEN         40          /**< minimum frame length */
#define MAX_CHANNELS          100         /**< maximal number of channels */

#define INIT_BUFFER_SIZE      10240  /**< Initial buffer size CD1 & CD11 */

#define LEN_CD1_PAF           32    /**< Size CD1.0 Port Assignment frame */
#define LEN_CD1_CRF            8    /**< Size CD1.0 Connection request frame */

#define LOOKUP_CD        12  /**< Bytes which are read from socket, but
                                  left in input buffer to determine frame
                                  type and size of needed frame buffer.
                                  Should be < 16 in both cases to avoid
                                  1 second sleeps (see readWithTimeOut) */
#define LOOKUP_CD1       12  /**< CD1.0 lookup size */
#define LOOKUP_CD11       8  /**< CD1.1 lookup size */


/* ==== Defines for Data and data Format Frames warnings and errors ========= */

#define NO_ERROR            0
#define ER_INTERNAL         (uint32_t)(1<<0)  /**< Internal parsing error */
#define WA_INVALID_CHANNEL  (uint32_t)(1<<1)  /**< invalid channel(s) in
                                                   frame */
#define WA_DF_SIZE          (uint32_t)(1<<2)  /**< invalid DF size */
#define AWA_NOMINAL_TIME    (uint32_t)(1<<3)  /**< Nominal time specifies
                                                   future time */
#define ER_DES_SIZE         (uint32_t)(1<<4)  /**< invalid description size */
#define WA_MAX_DF_SIZE      (uint32_t)(1<<5)  /**< invalid maxDF size in DFF
                                                   frame */
#define WA_CHANNEL          (uint32_t)(1<<6)  /**< invalid channel number
                                                   in DFF frame */
#define ER_DFF_SIZE         (uint32_t)(1<<7)  /**< invalid size of DFF frame */
#define WA_CRC              (uint32_t)(1<<8)  /**< invalid CRC, all (CD1.1)
                                                   frame event */
#define AWA_WARNING_CHANNEL (uint32_t)(1<<9)  /**< Always warning events
                                                   in channel */
#define ER_SIZE             (uint32_t)(1<<10) /**< invalid size, all frame
                                                   event */
#define ER_FRAME2BIG        (uint32_t)(1<<11) /**< frame to large, all
                                                   frame event */
#define ER_PROTOCOL         (uint32_t)(1<<12) /**< protocol violation,
                                                   e.g no DF, all frame event */
#define WA_NO_SIG           (uint32_t)(1<<13) /**< no frame signature,
                                                   all (CD1.1) frame event */
#define WA_INVALID_SIG      (uint32_t)(1<<14) /**< invalid signature, all
                                                   (CD1.1) frame event */
#define WA_NO_CERT          (uint32_t)(1<<15) /**< no certificate found,
                                                   all (CD1.1) frame event */
#define WA_UNSUPPORTED      (uint32_t)(1<<16) /**< Frame type not supported yet,
                                                   all frame event */
#define WA_NO_CERTS_CACHED  (uint32_t)(1<<17) /**< no certificate cached,
                                                   all frame event */
#define WA_AUTHENTICATION   (uint32_t)(1<<18) /**< channel authentication
                                                   failure */
#define WA_UNKNOWN_TYPE     (uint32_t)(1<<19) /**< Unknown frame type
                                                   (-->re-acquire) */
#define ER_NOT_PARSED       (uint32_t)(1<<20) /**< Frame is not parsed complete
                                                   (e.g. too short frame) */
#define WA_INVALID_ALERT_ID (uint32_t)(1<<21) /**< invalid alert type */
#define ER_STATION_NAME     (uint32_t)(1<<22) /**< invalid station name */
#define ER_COMMAND_SIZE     (uint32_t)(1<<23) /**< invalid command size */
#define ER_INVALID_CHANNEL  (uint32_t)(1<<24) /**< at least one channel has an
                                                   error condition */
#define ER_NO_CTRL_STATION  (uint32_t)(1<<25) /**< station not allowed to send
                                                   command requests */
#define ER_STR_SIZE         (uint32_t)(1<<26) /**< invalid channel string
                                                   size */
#define WA_FRAME_TIME_LEN   (uint32_t)(1<<27) /**< frame time length <= 0 */
#define ER_CMD_TOO_OLD      (uint32_t)(1<<28) /**< command frame too old */


/* macros for DF/DFF parsing warnings/errors */
#define HAS_CHANNEL_EVENT(a) (0 != ((a) & (AWA_WARNING_CHANNEL | \
                                           ER_INVALID_CHANNEL | \
                                           WA_INVALID_CHANNEL)))
#define HAS_DF_ERRORS(a) (0 != ((a) & (ER_INTERNAL | ER_DES_SIZE | ER_STR_SIZE \
                                       | ER_DFF_SIZE | ER_INVALID_CHANNEL \
                                       | ER_NOT_PARSED)))
#define HAS_DF_WARNINGS(a) (0 != ((a) & (WA_DF_SIZE | WA_MAX_DF_SIZE | \
                                         WA_CHANNEL | WA_INVALID_CHANNEL | \
                                         WA_FRAME_TIME_LEN)))
#define HAS_DF_AUTH_WARNINGS(a) (0 != ((a) & (WA_AUTHENTICATION | \
                                              WA_NO_CERTS_CACHED)))
#define HAS_ANY_DF_WARNING(a) (0 != ((a) & (WA_DF_SIZE | \
                                            WA_MAX_DF_SIZE | \
                                            WA_CHANNEL | WA_INVALID_CHANNEL | \
                                            WA_FRAME_TIME_LEN | \
                                            AWA_WARNING_CHANNEL | \
                                            AWA_NOMINAL_TIME)))

/* --- Channel errors -----*/

#define NO_ERROR            0
#define ER_PACKET_SIZE      (uint32_t)(1<<0)  /**< invalid packet length */
#define ER_END_REACHED      (uint32_t)(1<<1)  /**< end of DF reached */
#define AWA_TIME_STAMP      (uint32_t)(1<<2)  /**< time stamp specifies
                                                   future time */
#define ER_SAMPLES          (uint32_t)(1<<3)  /**< invalid nbr. of samples */
#define WA_AUTHSWITCH       (uint32_t)(1<<4)  /**< invalid authentication
                                                   switch */
#define WA_COMPRESSSW       (uint32_t)(1<<5)  /**< invalid compression
                                                   switch */
#define WA_NOT_ZEROED       (uint32_t)(1<<6)  /**< trailing bytes in
                                                   DFF subframe */
#define WA_CALIB_PER        (uint32_t)(1<<7)  /**< invalid calibration
                                                   period */
#define ER_AUTH_OFFSET      (uint32_t)(1<<8)  /**< invalid authentication
                                                   offset */
#define WA_OPTIONSW         (uint32_t)(1<<9)  /**< invalid option switch */
#define ER_STATUS_SIZE      (uint32_t)(1<<10) /**< invalid status size */
#define ER_DATA_SIZE        (uint32_t)(1<<11) /**< invalid channel data size */
#define AWA_STEIM_COMPRESS  (uint32_t)(1<<12) /**< unsupported compression
                                                   type */
#define WA_NO_SIG           (uint32_t)(1<<13) /**< channel is not signed */
#define WA_INVALID_SIG      (uint32_t)(1<<14) /**< invalid channel signature */
#define WA_NO_CERT          (uint32_t)(1<<15) /**< no certificate for
                                                   channel found */
#define WA_SAMPLES_CAN      (uint32_t)(1<<16) /**< invalid Canadian
                                                   compressed data */
#define AWA_ITYPE           (uint32_t)(1<<17) /**< unsupported data type */
#define ER_UNEXPECTED       (uint32_t)(1<<18) /**< Unexpected error */
#define WA_SUB_TIME_STAMP   (uint32_t)(1<<19) /**< subframe starts out of
                                                   frame boundary */
#define WA_CALIB_FAC        (uint32_t)(1<<20) /**< invalid calibration
                                                   period */
#define WA_NOT_IN_SAMPLE    (uint32_t)(1<<21) /**< subframe starts not within
                                           one sample of frame nominal time */
#define WA_INVALID_NAME     (uint32_t)(1<<22) /**< invalid site or channel
                                                   name */

#define IS_INVALID_CHANNEL(a) (0 != ((a) & (ER_PACKET_SIZE | \
                                           ER_END_REACHED | \
                                           ER_SAMPLES | ER_STR_SIZE | \
                                           ER_AUTH_OFFSET | ER_STATUS_SIZE | \
                                           ER_DATA_SIZE | ER_UNEXPECTED)))

/* ==== CD1.1 trailer ======================================================= */

/**
 * CD-1.1 frame trailer structure
 *
 * This structure is used to store values corresponding to a CD-1.1 frame
 * trailer information
 */
typedef struct _CDtrailer
{
   uint32_t authKeyIdentifier; /**< Identifier for private Key (only unique
                                    within a station realm */
   uint32_t signatureSize;     /**< Size of signature => 0/40/144 */
   /*@dependent@*/
   const char *signature;      /**< Pointer to signature in frame buffer */
   uint64_t crc;               /**< Frame checksum */
} CDtrailer_t;

#define SZ_SIGNATURE_DSA     40 /**< Length of DSA non ASN.1 coded signature */
#define SZ_SIGNATURE_EC_MAX 144 /**< Length of EC non ASN.1 coded signature
                                     571 bit key. Approximately keyLen/2*4/8 */
#define SZ_SIGNATURE_EC_MIN 56  /**< Length of EC non ASN.1 coded signature
                                     224 bit key (minimum for SHA224) */
#define SZ_SIGNATURE_MAX    SZ_SIGNATURE_EC_MAX  /**< Max. length non ASN.1
                                                      coded signature */
#define SZ_TRAILER_WO_SIG   16 /**< Length of CD-1.1 trailer without
                                    counting the signature */
#define SZ_TRAILER_MAX  (SZ_TRAILER_WO_SIG + SZ_SIGNATURE_MAX)
#define IS_VALID_SIG_SIZE(ii)  (SZ_SIGNATURE_DSA == (ii) ||\
          ((ii) >= SZ_SIGNATURE_EC_MIN && (ii) <= SZ_SIGNATURE_EC_MAX &&\
           0 == ((ii) % 4)))

/* ==== CD1.1 header ======================================================== */

#define SZ_FRAME_CREATOR       8
#define SZ_FRAME_DESTINATION   8

/**
 * CD-1.1 frame header structure
 *
 * This structure is used to store values corresponding to a CD-1.1 frame
 * header information
 */
typedef struct _CDheader
{
   uint32_t frameType;                          /**< Frame type */
   uint32_t trailerOffset;                      /**< Offset to trailer */
   char     creator[SZ_FRAME_CREATOR];          /**< Frameset information */
   char     destination[SZ_FRAME_DESTINATION];  /**< Frameset information */
   uint64_t sequenceNbr;                        /**< Sequence number */
   uint32_t series;                             /**< Not used */
   uint32_t keyID;              /**< not part of header, but needed later */
} CDheader_t;

#define SZ_HDR      36   /**< Size of CD1.1 header. Don't use sizeof!! */
#define MIN_SZ_CD11_FRAME    (SZ_HDR + SZ_TRAILER_WO_SIG + 4) /**< minimum
                              size of CD-1.1 frame ( empty AF) */

/* ==== CD1.1 Command Response ============================================== */

typedef struct _CDcomRes
{
   /*@dependent@*/
   const char *stationName;    /* 8 bytes station name */
   /*@dependent@*/
   const char *site;           /* 5 bytes site name */
   /*@dependent@*/
   const char *channel;        /* 3 bytes channel name */
   /*@dependent@*/
   const char *location;       /* 2 bytes location name */
   /*@dependent@*/
   const char *timeStamp;      /* 20 bytes time stamp */
   uint32_t commandSize;       /* unpadded length of command field */
   /*@dependent@*/
   const char *command;        /* command; padded to be divisible by 4 */
   uint32_t responseSize;      /* unpadded length of response field */
   /*@dependent@*/
   const char *response;       /* response; padded to be divisible by 4 */
} CDcomRes_t;

#define SZ_COMMRES_PL      56 /* Minimum size of command request payload */

/* ==== CD1.1 Command Request =============================================== */

typedef struct _CDcomReq
{
   /*@dependent@*/
   const char *stationName;    /* 8 bytes station name */
   /*@dependent@*/
   const char *site;           /* 5 bytes site name */
   /*@dependent@*/
   const char *channel;        /* 3 bytes channel name */
   /*@dependent@*/
   const char *location;       /* 2 bytes location name */
   /*@dependent@*/
   const char *timeStamp;      /* 20 bytes time stamp */
   uint32_t commandSize;       /* unpadded length of command field */
   /*@dependent@*/
   const char *command;        /* command; padded to be divisible by 4 */
} CDcomReq_t;

#define SZ_COMMREQ_PL      48 /* Minimum size of command request payload */
#define OFFSET_COMMAND     SZ_HDR + 44              /* Offset of command */

/* ==== CD1.1 connection request payload ==================================== */

typedef struct _CDconnectRequest
{
   uint16_t majorProtoReq;       /* requested major protocol version */
   uint16_t minorProtoReq;       /* requested minor protocol version */
   /*@dependent@*/
   const char *stationName;      /* station requesting the connection */
   /*@dependent@*/
   const char *stationType;      /* requester type IMS,IDC,NDC */
   /*@dependent@*/
   const char *serviceType;      /* TCP */
   uint32_t IPaddress;           /* IP address of requester */
   uint16_t port;                /* port address of requester */
   uint32_t IPaddress2nd;        /* reserved */
   uint16_t port2nd;             /* reserved */
} CDconnectRequest_t;

#define SZ_CONNRQ_PL    32
#define LNG_CRF         (SZ_HDR + SZ_CONNRQ_PL + SZ_TRAILER_MAX)

/* ==== CD1.1 option request payload ======================================== */

typedef struct _CDoption
{
   uint32_t optionType;         /* type of option request */
   uint32_t optionSize;         /* unpadded size of option request */
   /*@dependent@*/
   const char *optionString;    /* value of option request padded to be
                                   divisible by 4 */
} CDoptions_t;

#define MAX_OPTIONS    3

typedef struct _CDoptionRequest
{
   uint32_t optionCount;         /* number of option requests */
   CDoptions_t option[MAX_OPTIONS];
} CDoptionRequest_t;

/* ==== Defines and data structures for channel mask ======================== */

#define MAX_LEN_CHANNEL_MASK  (MAX_CHANNELS + 1)

/**
 * structure for a channel name cache entry
 *
 * The channel name cache entry structure contains the external site and
 * channel name, the internal site and channel name (if sensor info is
 *  available), the position in the channel name mask, and a link to the next
 *  channel name for that station.
 */

typedef struct _chName {
#define LEN_SC_NAME    15      /**< Max.: length of site + '/' + channel name */
#define MAX_NAMES_CACHED   3   /**< maximum number of different names for same
                                    channel */
   char  *scName[MAX_NAMES_CACHED]; /**< site/channel names */
   int16_t nbrOfNames;         /**< Number of names for this channel */
   uint16_t maskPos;            /**< position in the channel mask */
   /*@null@*/
   struct _sensor *chSensorInfo; /**< sensor information for channel, if any */
   /*@owned@*//*@null@*/
   void *chCacheExt;              /**< for application extensions */
   /*@owned@*//*@null@*/
   struct _chName *nextChannel;   /**< reference to next channel name in
                                       cache */
} chName_t;

/**
 * structure for a station specific channel name cache
 *
 * The channel name  cache structure contains all expected channels
 * for a given station. All channel name structures are linked to
 * a channel name cache chain.
 */

typedef struct _chNameCache {
   /*@dependent@*/
   void *pStationID;    /**< Link back to station. Must be void * to
                             prevent include dependencies problem */
   uint16_t  nrOfChannels;     /**< number of channels in the channel cache */
   /*@owned@*//*@null@*/
   struct _chName *firstCh;    /**< reference to first channel name in cache */
} chNameCache_t;


/* ==== Structure holding CD1/CD1.1 data channel information ================ */

typedef struct _CDdataChannel
{
   uint32_t    channelLength; /* not counting this field */
   uint32_t    authOffset;    /* Authentication Offset (only CD1.1) */
   char        authSwitch;    /* authentication 0 -> off, 1 -> on */
   char        compression;   /* 0 -> off, 1 -> Canadian before signing
                                 2 -> Canadian after signing, 3 -> Steim
                                 before and 4 -> Steim after signing;
                                 Canadian values are swapped during parsing
                                 for CD1.1!! */
   char        sensorType;    /* unused in CD1 */
   char        option;        /* not in CD1 frame, but set in CD1 and CD1.1*/
   /* Next 3 fields must have at least a size of 16 bytes (CD1 channel name) */
#define LEN_SITE_NAME        6
   char        siteName[LEN_SITE_NAME + 1];     /* wfdisc field size + 1 */
#define LEN_CHANNEL_NAME     8
   char        channelName[LEN_CHANNEL_NAME + 1];  /* wfdisc field size + 1*/
   char        locationName[3];  /* CD1.1 size + 1 */
   char        dataFormat[3];

   float       calibFactor;
   float       calibPeriod;   /* channel descr. end */
   /*@dependent@*//*@null@*/
   const char  *pTimeStamp;   /* Only CD1.1 */
   double      timeStamp;     /* creation time of first sample; nbr of
                                 seconds and milliseconds from 1.1.1970 */
   uint32_t    subFrameTimeLng;  /* time in ms spawned by this data */
   uint32_t    nbrOfSamples;
   uint32_t    channelStatusSize;  /* Always 4 for CD1 */
   /*@dependent@*/
   const char  *channelStatus;
   uint32_t    dataSize;
   /*@dependent@*/
   const char  *channelData;
   uint32_t    dataOffset;       /* Offset of sample data in current DF */
   uint32_t    channelOffset;    /* Offset of channel subframe */
   uint32_t    subframeCount;
   uint32_t    authKeyIdentifier; /* Serial number in certificate, CD1.1 only */
   uint32_t    signatureSize;     /* Size of signature => 0/40/144 */
   /*@dependent@*//*@null@*/
   const char  *authString;
#define AUTH_FAILED       ((uint16_t)'F')   /* Signature check failed */
#define AUTH_PASSED       ((uint16_t)'V')   /* Signature check successful */
#define AUTH_UNKNOWN      ((uint16_t)'C') /* Signature not checked. Switched
                                             off or compiled without OpenSSL */
#define AUTH_SKIPPED      ((uint16_t)'S')   /* Skipped by user term. req. */
#define AUTH_NOT_CHECKED  ((uint16_t)'N')   /* NONE directive */
#define AUTH_NO_KEY       ((uint16_t)'K')   /* No key found */
#define AUTH_NO_SIG       ((uint16_t)'G')   /* Data not signed */
   uint16_t authentPass;     /* result of data authentication (see above) */
   uint32_t checkPass;       /* channel has passed validation check */
   int posInChMask;          /* position in channel mask or -1 */
} CDdataChannel_t;

/* ==== Structure holding CD1 DFF and/CD1.1 data frame information ========== */

typedef struct _CDdata
{
   uint32_t frameType;       /* 1 or 2 for CD1, 5 for CD1.1 */
   uint32_t frameLength;     /* Length of frame */
   uint32_t maxDFsize;       /* Max. length of Data Frame (only CD1) */
   uint32_t nbrOfChannels;   /* Number of Channels in this frame */
   uint32_t parsedDFFchannels;   /* Number of parsed channels in current DFF */
   uint32_t frameTimeLength; /* time in ms this frame encompasses */
   /*@dependent@*//*@null@*/
   const char  *pNominalTime; /* Only CD1.1 */
   double   nominalTime;     /* Nominal UTC start time of all channels */
   uint32_t channelStringSize;   /* Unpadded!! */
   CDdataChannel_t channel[MAX_CHANNELS];  /* Channel Data */
   uint32_t parsedChannels;   /* Number of parsed channels in current DF */
   uint32_t desLength;       /* Description Length in current CD1 DF */
   /*@dependent@*/
   const char *description;  /* Pointer to description (only CD1) */
   uint32_t DFFsaved;        /* flag to indicate if a DFF should be written
                                (used by senderAPI) */
   char channelMask[MAX_LEN_CHANNEL_MASK]; /* channel mask string */
} CDdata_t;

/* ==== CD1 Alert Frame  ==================================================== */

typedef struct _CD1alert
{
   uint32_t frameType;       /* Frame type */
   uint32_t frameLength;     /* Length of frame */
#define  NO_ACTION          0
#define  TERMINATE          1     /* by sender */
#define  FLUSH_FORMAT       2
#define  TERMINATE_REC   1000
   uint32_t alertType;       /* Type of alert */
} CD1alert_t;

#define SZ_CD1_ALERT    12   /* Size of CD1 alert frame */
#define MIN_SZ_CD1_FRAME   SZ_CD1_ALERT

/* ==== CD1.1 AckNack payload  ============================================== */

#define MAX_GAPS 100

typedef struct _CDgaps
{
   uint64_t gapBegin;   /**< lowest missing sequence number */
   uint64_t gapEnd;     /**< highest missing sequence number */
} CDgaps_t;

typedef struct _CDframeset
{
   struct _CDframeset *nextFrameset;    /**< Pointer to next frame set */
#define MAX_FRAMESET_NAME  20
   char name[MAX_FRAMESET_NAME]; /**< Name e.g. "KDC:0"   */
   uint64_t lowestSeqNbr;      /**< lowest sequence number received or 0 */
   uint64_t highestSeqNbr;     /**< highest sequence number received or -1 */
   uint32_t gapCount;          /**< Number of subsequent gaps */
   CDgaps_t gaps[MAX_GAPS];    /**< Subsequent gaps */
} CDframeset_t;

#define CREATE_FS    1       /**< Create frame set if not found */
#define SEARCH_FS    2       /**< Only search for frame set */
#define MAP_IRL_2FS  4       /**< Map IRL representation to frame set name
                                  before search */

typedef struct _CDframeSets
{
   struct _CDframeset *firstFrameset;    /**< Pointer to first frame set */
   uint32_t fsCount;          /**< Number of known frame sets */
} CDframeSets_t;

#define MAX_SIZE_ACKNACK_FRAME  (SZ_HDR + 40 + (16 * MAX_GAPS) + SZ_TRAILER)

/* ==== Sensor Information ================================================== */

typedef struct _sensor  /**< Describes a sensor */
{
   char station[LEN_SITE_NAME+1];     /**< station name */
   char stream[LEN_SITE_NAME+1];     /**< stream name */
   char intern_sta[LEN_SITE_NAME+1];     /**< internal site name */
   char extern_sta[LEN_SITE_NAME+1];     /**< external site name */
   char intern_chan[LEN_CHANNEL_NAME+1]; /**< channel name from the frame.
                                              Must match intern_chan
                                              in channame and chan in
                                              sensor table in DB mode */
   long chanID;                          /**< Channel ID from DB */
#define LEN_INSTRUMENT     6
   char insType[LEN_INSTRUMENT+1];       /**< instrument type */
   float nomCalib;                /**< nominal calibration value */
   float nomCalibPer;             /**< nominal calibration period */
   float expSampleRateInDB;       /**< expected sample rate from DB */
   double startTime;              /**< record is valid from  start time */
   double endTime;                /**< record is valid until end time */
   int revision;                  /**< channel name revision */
   int position;                  /**< number of channel in mask column */
} sensor_t;

/* ===== Function prototypes ================================================ */

extern void parseHeader(const char *buffer, /*@out@*/ CDheader_t *header);
extern void setHeader(char *buffer, const uint32_t frameType,
                      const uint32_t trailerOffset, const char *creator,
                      const char *destination, const uint64_t sequenceNbr);
extern void parseCRframe(const char *buffer,
                         /*@out@*/ CDconnectRequest_t *payload);
extern void setCRpayload(char *buffer, const char *responderName,
                         const char *responderType, const char *IPaddr,
                         const uint16_t port);
extern void parseTrailer(const char *buffer, /*@out@*/ CDtrailer_t *trailer);

extern void parseAF(const char *buffer, /*@out@*/ CD1alert_t *frame);
extern uint32_t setTrailer(char *buffer, const uint32_t offset,
                           const uint32_t authKeyID,
                           /*@null@*/ const char *signature, size_t sigSize);

extern uint32_t parseAndCheckDFF(/*@null@*/CDdata_t *frame,
                                 uint32_t strictMode,
                                 const char *buffer,
                                 chNameCache_t *chNameCache);
extern uint32_t parseAndCheckDataPayload(/*@null@*/CDdata_t*frame,
                                         uint32_t strictMode,
                                         const char *buffer,
                                         uint16_t maxPosDeviation,
                                         uint32_t frameLen,
                                         long nowtime,
                                         chNameCache_t *chNameCache);
extern uint32_t parseAndCheckDF(/*@null@*/CDdata_t*frame, uint32_t strictMode,
                                const char *buffer, uint16_t maxPosDeviation,
                                long nowtime,
                                chNameCache_t *chNameCache);
/*@dependent@*/ /*@null@*/ extern CDframeset_t *
getFrameSet(const char *framesetName, CDframeSets_t *pFs, const int mode);
extern void buildFramesetName(/*@out@*/char *frameSetName, size_t maxLen,
                       const char *creator, const char *destination);
/*@dependent@*//*@null@*/extern CDframeset_t *
parseAckNack(const char *buffer, uint32_t len, CDframeSets_t *pFrameSets,
             uint16_t *event);
extern void irl2frameSet(const char *from, /*@out@*/ char *to, size_t to_size);
extern uint32_t setAckNakPayload(/*@out@*/char *buffer, CDframeset_t *frameset,
                                 const char *creator, const char *destination);
extern uint32_t setCORpayload(/*@out@*/char *buffer, const char *stationName,
                         const char *site, const char *channel,
                         const char *location, const char *command);
extern void parseComReq(/*@dependent@*/ const char *buffer,
                        /*@out@*/ CDcomReq_t *payload);
extern uint32_t setCommandResponseSize(char *buffer, uint32_t lenOfMessage);
extern int parseComRes(/*@dependent@*/const char *buffer,
                       /*@out@*/ CDcomRes_t *payload, uint32_t bufferSize);
extern int parseOptReq(/*@dependent@*/const char *buffer,
                       /*@out@*/ CDoptionRequest_t *payload,
                       const uint32_t offsetTrailer);
extern int checkCD1frameType(const uint32_t *intBuffer,
                             /*@out@*/double *const nominalTime);
extern int checkCD11frameType(const uint32_t *intBuffer,
                              /*@out@*/double *const nominalTime);
extern void setCD1ConRq(/*@out@*/char *buffer, const char *pName);
extern void setCD1AF(/*@out@*/char *bufferAF, uint32_t alertType);
extern uint32_t parseAndCheckCD11dataChannel(CDdataChannel_t *channel,
                                             uint32_t strictMode,
                                             const char *buffer,
                                             uint16_t maxPosDeviation,
                                             long nowTime,
                                             uint32_t actOffset,
                                             uint32_t maxLength,
                                             double nominalTime,
                                             int checkTimeStamp,
                                             chNameCache_t *chNameCache);
extern uint16_t checkORFpayload(CDheader_t *header, char *buffer,
                                uint32_t strictMode);
extern int getChannelMask(CDdata_t *frame, uint32_t nbrOfExpectedChannels);
extern int getChMaskPosition(char *siteName, char *channelName,
                             chNameCache_t *chNameCache, /*@out@*/int *rFlag,
                             uint32_t strictMode);
extern int checkChMaskPosition(char *siteName, char *channelName,
                               chNameCache_t *chNameCache);
/*@null@*//*@dependent@*/extern chName_t *
getChannelEntryByIndex(uint16_t chIndex, chNameCache_t *chNameCache);
extern uint16_t checkIfFrameValid(char *buff, char clfType, uint32_t clfSize,
                                  unsigned char sType);
#endif /* _FRAMES_H */
