#ifndef _SENDERAPI_H
#define _SENDERAPI_H

/**
 * @internal
 * @author    Gerald Klinkl
 *
 * Revision 1.13  2014-11-10 GK
 * Add and update function references for doxygen
 *
 * Revision 1.12  2013-04-03 gerald
 * Fix include dependencies
 *
 */

#include "cd_common.h"

/* ==== Defines and data structures ========================================= */
/**
 * @ingroup libCD
 * @addtogroup cdSenderAPI
 *
 * @{
 *
 */

/**
 * @short Channel data description structure
 *
 * The channel data description structure is used to provide channel
 * description for channels
 */

typedef struct _channelDescr
{
   /*@dependent@*/
   char *siteNameAPI; /**< site name */
   /*@dependent@*/
   char *channelNameAPI; /**< channel name */
   /*@dependent@*/
   char *locationNameAPI; /**< location name (only CD-1.1) */
   /*@dependent@*/
   char *datatypeAPI; /**< uncompresses data format (only CD-1.1) */
   char  sensorTypeAPI; /**< (only CD-1.1) 0=seismic;
                                         1=hydroacustic;
                                         2=infrasonic;
                                         3=weather;
                                         >3 other */
   char  optionFlagAPI; /**< 0=unused;1=calib and calper provided */
   char  authFlagAPI;   /**< 0=off;1=on */
   char  compressionAPI; /**< transformation:
                         0 -> off,
                         1 -> Canadian before signing (only CD-1.1)
                         2 -> Canadian after signing,
                         3 -> Steim before signing (not supported)
                         4 -> Steim after signing (not supported) */
   float calibFactorAPI; /**< calibration factor */
   float calibPeriodAPI; /**< calibration period */

} channelDescr_t;

/**
 * @short Channel data structure
 *
 * The channel data structure is used to provide channel data
 * to the API
 */

typedef struct _channelData
{
   /*@dependent@*/
   const char *channelDataAPI; /**< buffer containing channel data */
   uint32_t channelDataSizeAPI; /**< channel buffer size */
   double timeStampAPI; /**< channel data creation time in seconds and
                           milliseconds since 1.1.1970 00:00 */
   uint32_t subframeTimeLengthAPI; /**< time in milliseconds spawned by this
                                   channel data */
   uint32_t nbrOfSamplesAPI; /**< number of samples; 0 if a complete CSF is
                                provided in channelDataAPI */
   uint32_t channelStatusSizeAPI; /**< Always 4 for CD1  */
   /*@dependent@*/
   const char *channelStatusDataAPI; /**< status data for channel */
   uint32_t subframeCountAPI; /**< subframe count as assigned by digitizer */
   uint32_t authKeyIdentifierAPI;/**< serial number in certificate only CD1.1*/
   uint32_t signatureSizeAPI; /**< size of signature >= 40 */
   /*@dependent@*//*@null@*/
   const char *signatureAPI; /**< channel signature/authentication */

} channelData_t;

/**
 * @}
 */

/**
 * Channel reference data structure
 *
 * The Channel reference data structure is used to keep track
 * of channels provided by the user, internal used buffers for
 * channel data and data signing callback functions
 */

typedef struct _channelRef
{
   int nbrOfChannels;    /**< number of channels set so far */
   int dataFormatChanged;          /**< flag that data format has changed */
   char isProvided[MAX_CHANNELS];  /**< indicator if the channel data have
                                      been set */
   char oldProvided[MAX_CHANNELS]; /**< indicator if the channel data have
                                      been set in previous frame */
   channelDescr_t channelDes[MAX_CHANNELS]; /**< channel descriptions buffer */
   channelData_t channelData[MAX_CHANNELS]; /**< channel data buffer */
   /*@only@*/
   char *cachePointer[MAX_CHANNELS]; /**< cache buffers for channel data */
   /*@null@*/
   int (*callBackSign)(const char *data, size_t dataLen,
                       /*@out@*/ char *signature,
                       struct _channelRef *pChannelref, size_t *sigLen);
              /**< pointer to application specific function which is
                   called before a frame is saved */
   /*@null@*/
   void *pExtPtr;  /**< channelRef extension pointer */
} channelRef_t;


/* ==== Function prototypes ================================================= */

/*@null@*//*@dependent@*/
extern station_t *openBuffer(const char *configFile, char *stationName,
                              unsigned char stationType);
extern int writeFrame(station_t *pStation, uint32_t desLength,
                      const char *description);
extern int initChannel(station_t *pStation, channelDescr_t *pChannelDescr);
extern int setChannelData(station_t *pStation, int channelID,
                          channelData_t *pChannelData);
extern int setChannelSubFrame(station_t *pStation, int channelID,
                              const char *pCSF, uint32_t lenCSF,
                              uint32_t timeLengthCSF);
extern int clearChannelInfo(station_t *pStation);
extern int checkChannelIsPresent(station_t *pStation,
                                 channelDescr_t *pChannelDescr);
extern int checkChannelStartTime(station_t *pStation,
                channelDescr_t *pChannelDescr, double startTime,
                uint32_t CSFTimeLength, double oneSamp);

#endif /* _SENDERAPI_H */
