#ifndef _IDCODBC_H
#define _IDCODBC_H

/*******************************************************/
/*  International Data Centre                          */
/*  Comprehensive Test Ban Treaty Organization         */
/*  Vienna                                             */
/*  Austria                                            */
/*******************************************************/

/**
 * @author    Gerald Klinkl
 *
 * $Log: idcodbc.h,v $
 * Revision 1.15  2014-01-07  gerald
 * Add initial support for prepared statements. No wide characters,
 * complex SQL date types or stored procedures are supported!
 * Add and update comments for man pages.
 */

#include <inttypes.h> /* To get fixed size (64/32 bits) integer types */
#include <sql.h>
#include <sqlext.h>
#include <sqltypes.h>

#define ODBC_STRLEN 128+1
#define ODBC_REMLEN 254+1

#define NO_AUTO_COMMIT 0  /**< Switch off auto commit */
#define AUTO_COMMIT 1  /**< Switch on auto commit */

/* Error codes ODBCGet* functions */
#define ODBC_NO_ERROR        0
#define ODBC_NOT_FOUND       1   /**< No column with this name found */
#define ODBC_VALUE_OVERFLOW  2   /**< The resulting value was out of range */
#define ODBC_TYPE_MISMATCH   4   /**< The resulting value has not the requested
                                       type (e.g. double instead of integer) */
#define ODBC_PARAM_MISMATCH  8   /**< Number of bind parameters do not match
                                      number of question marks */
#define ODBC_UNSUPPORTED    16   /**< Unsupported C-data type */
#define ODBC_ERROR          -1

enum ODBCbindAttr {
   ODBC_GET_ATTRIBUTES = 0,  /**< try to get variable type from SQL driver */
   ODBC_HAVE_ATTRIBUTES = 1, /**< variable type is supported by caller */
   ODBC_PREPARE_ONLY = 2     /**< prepare only SQL, no binding of variables */
};

/* ==== Data structures ===================================================== */

/**
 * @addtogroup ODBC_Result
 *
 * @short ODBC result structure
 *
 * This structure is used to hold the result of an arbitrary SQL query.
 * If the query was a SELECT statement, it contains an array of
 * ODBC_ColumnResult structures, which describe the columns and which
 * contain variables to read the result of the SELECT statement. An
 * ODBC result structure has to be released using ODBCFreeResult() if
 * the result of an arbitrary SQL query is not longer needed. If the SQL
 * query was a prepared statement, ODBCFreeResult() releases also the
 * prepared statement. To execute a prepared SELECT statement more than
 * once, the application must call ODBCCloseCursor() before re-executing
 * the SELECT statement.
 *
 * @see ODBCExecDirect(), ODBCExec(), ODBCPrepare(), ODBCFreeResult(),
 *      ODBCFetchNextRow(), ODBCCloseCursor()
 *
 * @{
 */

/**
 * @short ODBC result structure for one column
 *
 * This structure describes one column and is also used to bind
 * variables to column results. This structure should not be set or
 * modified manually.
 */

typedef struct _ODBC_ColumnResult
{
   SQLCHAR      columnName[ODBC_STRLEN]; /**< Column name */
   SQLSMALLINT  nameLength;              /**< Length of column name */
   SQLSMALLINT  dataType;                /**< SQL data type of column */
   SQLULEN      columnSize;              /**< Precision as defined in DB */
   SQLSMALLINT  decimalDigits;           /**< Scale of the column */
   SQLSMALLINT  nullable;                /**< Whether null values are allowed */
   SQLLEN       colDispSize;             /**< Maximum bytes needed to display
                                              data in character form */
   SQLLEN       outLen;                  /**< Bytes needed to display data in
                                              character form when read */
   union {
       /*@null@*/SQLCHAR     *sql_String;  /**< Bind as character string */
       SQLDOUBLE   sql_Double;             /**< Bind as double */
       SQLINTEGER  sql_Long;               /**< Bind as long integer */
       SQLSMALLINT sql_Int;                /**< Bind as integer */
       SQL_TIMESTAMP_STRUCT sql_DateTime;  /**< Bind as datetime */
   } bindTarget; /**< Union for bind target */
} ODBC_ColumnResult;

/**
 * @short ODBC result structure
 *
 * @ingroup ODBCGetAs
 * @ingroup ODBCExec
 *
 * This structure is used to hold the result of an arbitrary SQL query.
 */

typedef struct _ODBC_Result
{
   /*@dependent@*//*@null@*/
   void *selfPrt;             /**< Pointer to start of structure */
   /*@null@*/SQLHSTMT hstmt;  /**< Statement handle */
   SQLSMALLINT numColumns;    /**< Number of columns in result */
   int      actRow;           /**< Current row number */
   /*@null@*/ODBC_ColumnResult *resCols;  /**< Pointer to ODBC_ColumnResults */
   SQLRETURN sqlRetCode;      /**< Statement return code */
   int resState;              /**< State of result structure */
#define ODBC_RES_FREED     0  /**< ODBC result has been freed */
#define ODBC_RES_PREPARED  1  /**< has prepared statement */
} ODBC_Result;


/**
 * @short Structure to request the result of an SQL SELECT statement
 *
 * @ingroup ODBCGetAs
 * @ingroup ODBCExec
 *
 * This structure is used to a read the content of a specified column from the
 * current rowset. The column is specified by the c_name field, which contains
 * the name of the field as used in the table definition. c_p4value is
 * a reference to a variable where the value of the column should be stored.
 * The C data type of *c_p4value is defined by c_Type, and c_Size is the
 * storage size of the variable to store the value. c_pIndex is the index
 * the column in the rowset and must be set to -1 at the first time the column
 * is read. Functions which read the column values will set the appropriate
 * index for subsequent calls.
 */

typedef struct _reqColumns
{
  const char *c_name;        /**< name of column to read */
  /*@shared@*//*@reldef@*/ void *c_p4value; /**< referenced variable
                                              to store value */
  int c_Type;               /**< C data type of p4value */
#define CT_INT     1        /**< signed integer */
#define CT_LONG    2        /**< signed long integer */
#define CT_INT16U  3        /**< 16 bit unsigned integer */
#define CT_INT32U  4        /**< 32 bit unsigned integer */
#define CT_INT32S  5        /**< 32 bit signed integer */
#define CT_INT64S  6        /**< 64 bit signed integer */
#define CT_FLOAT   7        /**< IEEE float */
#define CT_DOUBLE  8        /**< IEEE double */
#define CT_STRING  9        /**< String */
#define CT_SHORT  10        /**< Signed short integer */
#define CT_SHORTU 11        /**< Signed short integer */
#define CT_LONGU  12        /**< Unsigned long integer */
  size_t c_Size;            /**< storage size of c_p4value */
  int c_pIndex;             /**< Index of column to read; initial value must
                                 be -1 */
} reqColumns_t;

/**
 * @short Structure to bind parameter values for prepared SQL statements
 *
 * @ingroup ODBCBind
 * @ingroup ODBCExec
 *
 * This structure is used to bind parameters to a prepared SQL statement.
 * The structure member @e b_name should be supplied with the database name
 * of the related parameter. Is is used to read the column description
 * from the database driver, which is needed to bind the parameter.
 * @e b_p4bind is a pointer to a buffer that, when ODBCExec() is called,
 * contains the actual data for the parameter. @e b_Type specifies the C
 * data type of the parameter to bind. @e b_strlenOrInd is  a multi purpose
 * field which could be set to SQL_NULL_DATA if the parameter value is
 * NULL, SQL_NTS if the parameter value is a null terminated string or
 * to the length of a binary (non null terminated) string. This value is
 * ignored if the parameter is not a string or not set to SQL_NULL_DATA.
 * The remaining structure members are only used when ODBCPrepare() is
 * called with mode not set to 0. @e b_SQLDataType is the SQL data type of
 * the parameter, @e b_columnSize is the size of the column or expression
 * of the corresponding parameter marker, @e b_decimalDigits is the number
 * of decimal digits of the column or expression of the corresponding
 * parameter marker and @e b_bufferLength is the length of the @e b_p4bind
 * buffer in bytes. For more information about these structure members, see
 * SQLBind() in the ODBC documentation.
 * The application is responsible to set the correct database field
 * name in @e b_name, libidcodbc does not check if the name is set correct.
 *
 */

typedef struct _bindColumns
{
   const char *b_name;    /**< name of database field */
   /*@shared@*//*@reldef@*/ void *b_p4bind; /**< referenced variable
                                                 to provide current value */
   int b_Type;            /**< C data type of p4bind, see reqColumns_t.
                               Due to limitation in the ODBC API,
                               integers must be bind as SQL_BIGINT */
   SQLLEN       b_strlenOrInd;  /**< length of content if p4value is a non
                                     null-terminated string, SQL_NTS if it is
                                     a null-terminated string or SQL_NULL_DATA
                                     if the parameter value is NULL,
                                     otherwise ignored */
   SQLSMALLINT  b_SQLDataType;   /**< SQL data type of column */
   SQLULEN      b_columnSize;    /**< Precision as defined in DB */
   SQLSMALLINT  b_decimalDigits; /**< Scale of the column */
   SQLLEN       b_bufferLength; /**< Size of parameter buffer */
} bindColumns_t;

/**
 * @short Structure to describe database type
 *
 * @ingroup ODBCExec
 *
 * This structure is used to describe the database used
 */

typedef struct _DBdescription_t
{
   const char *DBServerString;  /**< DB name as string */
   const int DBServerName;      /**< DB ID used internally */
#define DB_UNKNOWN   -1
#define DB_ORACLE     0
#define DB_POSTGRES   1
#define DB_MYSQL      2
#define DB_SQLITE     3
   const char *selectOption;    /**< Select for update option */
   const int DBServerType;      /**< Type of DB server, see below */
#define DB_NEED_START    1     /**< DB needs start transaction  */
#define DB_NO_LOCKING    2     /**< No 'select for update' support */
   /*@NULL@*/const char *opening;    /**< Command before select for update */
} DBdescription_t;

/* @} */

/* ==== Prototypes ========================================================== */

extern void ODBCInitLogs(const char *syslogID, int debugLevel);
#define ODBCErrMsg() ""
#define ODBCConnect(ds,usr,pw,ac) ODBCConnect2(ds,usr,pw,ac,30)
extern /*@only@*//*@null@*/SQLHDBC ODBCConnect2(const char *dataSource,
                            const char *user,
                            const char *passwd, int autoCommit,
                            int loginTimeout);
#define ODBCGeneralQuery(dh, qu, re) ODBCExecDirect(dh, qu, re)
extern int ODBCExecDirect(SQLHDBC hdbc, const char *query,
                            /*@out@*/ODBC_Result *result);
extern int ODBCCommit(SQLHDBC hdbc);
extern int ODBCRollBack(SQLHDBC hdbc);
extern int ODBCFetchNextRow(ODBC_Result *result);
extern int ODBCGetAsInteger(ODBC_Result *result, const char *columnName,
                            /*@out@*/int *value, /*@null@*/int *pIndex);
extern int ODBCGetAsLong(ODBC_Result *result, const char *columnName,
                         /*@out@*/long *value, /*@null@*/int *pIndex);
extern int ODBCGetAsInt32(ODBC_Result *result, const char *columnName,
                          /*@out@*/int32_t *value, /*@null@*/int *pIndex);
extern int ODBCGetAsUint32(ODBC_Result *result, const char *columnName,
                           /*@out@*/uint32_t *value, /*@null@*/int *pIndex);
extern int ODBCGetAsUint16(ODBC_Result *result, const char *columnName,
                           /*@out@*/uint16_t *value, /*@null@*/int *pIndex);
extern int ODBCGetAsInt64(ODBC_Result *result, const char *columnName,
                          /*@out@*/int64_t *value, /*@null@*/int *pIndex);
extern int ODBCGetAsFloat(ODBC_Result *result, const char *columnName,
                          /*@out@*/float *value, /*@null@*/int *pIndex);
extern int ODBCGetAsDouble(ODBC_Result *result, const char *columnName,
                           /*@out@*/double *value, /*@null@*/int *pIndex);
extern int ODBCGetAsString(ODBC_Result *result, const char *columnName,
                           /*@out@*/char *outStr, size_t strSize,
                           /*@null@*/int *pIndex);
extern int ODBCGetColumns(size_t tableSize, reqColumns_t columns2read[],
                          ODBC_Result *pResult);
extern void ODBCFreeResult(ODBC_Result *result);
extern void ODBCCloseCursor(ODBC_Result *result);
extern int ODBCDisconnect(SQLHDBC hdbc);
extern int ODBCDumpRow(FILE *fp, ODBC_Result *result);
extern int ODBCSetRequestIdIncrement(const char *keyname, int idIncrement);
extern long ODBCGetNextId(SQLHDBC hdbc, const char *tableName,
                          const char *keyname);
extern int ODBCRecycleIds(const char *keyname, long *ids, int numReturned);
extern int ODBCNumberIDsAvailable(const char *keyname);
extern int simulateTableLock(SQLHDBC hdbc,
                             const char *tableName); /* for use in odbctest.c */
extern int ODBCPrepare(SQLHDBC hdbc, const char *query, size_t tableSize,
                       /*@null@*/bindColumns_t columns2bind[],
                       /*@out@*/ODBC_Result *result, int mode);
extern int ODBCBindParams(SQLHDBC hdbc, size_t tableSize,
                       /*@null@*/bindColumns_t columns2bind[],
                       ODBC_Result *result, int mode);
extern int ODBCExec(SQLHDBC hdbc, /*@out@*/ODBC_Result *result);
extern /*@dependent@*/DBdescription_t * ODBCGetDBdescription(SQLHDBC hdbc);
extern /*@observer@*/ const char *libIDCodbcVersion(void);
#endif /* _IDCODBC_H */
